package com.tinkerforge;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickServoActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Enable")
    public void enable(
            @ActionInput(name = "servoNum") short servoNum) throws TinkerforgeException {
        ((BrickServo)this.handler.getDevice()).enable(servoNum);
    }

    public static void enable(@Nullable ThingActions actions, short servoNum) throws TinkerforgeException {
        if (actions instanceof BrickServoActions) {
            ((BrickServoActions) actions).enable(servoNum);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickServoActions class.");
        }
    }

    @RuleAction(label = "Disable")
    public void disable(
            @ActionInput(name = "servoNum") short servoNum) throws TinkerforgeException {
        ((BrickServo)this.handler.getDevice()).disable(servoNum);
    }

    public static void disable(@Nullable ThingActions actions, short servoNum) throws TinkerforgeException {
        if (actions instanceof BrickServoActions) {
            ((BrickServoActions) actions).disable(servoNum);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickServoActions class.");
        }
    }

    @RuleAction(label = "Is Enabled")
    public @ActionOutput(name = "enabled", type="boolean")
           Map<String, Object> isEnabled(
            @ActionInput(name = "servoNum") short servoNum) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        boolean value = ((BrickServo)this.handler.getDevice()).isEnabled(servoNum);
        result.put("enabled", value);
        return result;
    }

    public static Map<String, Object> isEnabled(@Nullable ThingActions actions, short servoNum) throws TinkerforgeException {
        if (actions instanceof BrickServoActions) {
            return ((BrickServoActions) actions).isEnabled(servoNum);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickServoActions class.");
        }
    }

    @RuleAction(label = "Set Position")
    public void setPosition(
            @ActionInput(name = "servoNum") short servoNum,
            @ActionInput(name = "position") short position) throws TinkerforgeException {
        ((BrickServo)this.handler.getDevice()).setPosition(servoNum, position);
    }

    public static void setPosition(@Nullable ThingActions actions, short servoNum, short position) throws TinkerforgeException {
        if (actions instanceof BrickServoActions) {
            ((BrickServoActions) actions).setPosition(servoNum, position);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickServoActions class.");
        }
    }

    @RuleAction(label = "Get Position")
    public @ActionOutput(name = "position", type="short")
           Map<String, Object> getPosition(
            @ActionInput(name = "servoNum") short servoNum) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        short value = ((BrickServo)this.handler.getDevice()).getPosition(servoNum);
        result.put("position", value);
        return result;
    }

    public static Map<String, Object> getPosition(@Nullable ThingActions actions, short servoNum) throws TinkerforgeException {
        if (actions instanceof BrickServoActions) {
            return ((BrickServoActions) actions).getPosition(servoNum);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickServoActions class.");
        }
    }

    @RuleAction(label = "Get Current Position")
    public @ActionOutput(name = "position", type="short")
           Map<String, Object> getCurrentPosition(
            @ActionInput(name = "servoNum") short servoNum) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        short value = ((BrickServo)this.handler.getDevice()).getCurrentPosition(servoNum);
        result.put("position", value);
        return result;
    }

    public static Map<String, Object> getCurrentPosition(@Nullable ThingActions actions, short servoNum) throws TinkerforgeException {
        if (actions instanceof BrickServoActions) {
            return ((BrickServoActions) actions).getCurrentPosition(servoNum);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickServoActions class.");
        }
    }

    @RuleAction(label = "Set Velocity")
    public void setVelocity(
            @ActionInput(name = "servoNum") short servoNum,
            @ActionInput(name = "velocity") int velocity) throws TinkerforgeException {
        ((BrickServo)this.handler.getDevice()).setVelocity(servoNum, velocity);
    }

    public static void setVelocity(@Nullable ThingActions actions, short servoNum, int velocity) throws TinkerforgeException {
        if (actions instanceof BrickServoActions) {
            ((BrickServoActions) actions).setVelocity(servoNum, velocity);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickServoActions class.");
        }
    }

    @RuleAction(label = "Get Velocity")
    public @ActionOutput(name = "velocity", type="int")
           Map<String, Object> getVelocity(
            @ActionInput(name = "servoNum") short servoNum) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickServo)this.handler.getDevice()).getVelocity(servoNum);
        result.put("velocity", value);
        return result;
    }

    public static Map<String, Object> getVelocity(@Nullable ThingActions actions, short servoNum) throws TinkerforgeException {
        if (actions instanceof BrickServoActions) {
            return ((BrickServoActions) actions).getVelocity(servoNum);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickServoActions class.");
        }
    }

    @RuleAction(label = "Get Current Velocity")
    public @ActionOutput(name = "velocity", type="int")
           Map<String, Object> getCurrentVelocity(
            @ActionInput(name = "servoNum") short servoNum) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickServo)this.handler.getDevice()).getCurrentVelocity(servoNum);
        result.put("velocity", value);
        return result;
    }

    public static Map<String, Object> getCurrentVelocity(@Nullable ThingActions actions, short servoNum) throws TinkerforgeException {
        if (actions instanceof BrickServoActions) {
            return ((BrickServoActions) actions).getCurrentVelocity(servoNum);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickServoActions class.");
        }
    }

    @RuleAction(label = "Set Acceleration")
    public void setAcceleration(
            @ActionInput(name = "servoNum") short servoNum,
            @ActionInput(name = "acceleration") int acceleration) throws TinkerforgeException {
        ((BrickServo)this.handler.getDevice()).setAcceleration(servoNum, acceleration);
    }

    public static void setAcceleration(@Nullable ThingActions actions, short servoNum, int acceleration) throws TinkerforgeException {
        if (actions instanceof BrickServoActions) {
            ((BrickServoActions) actions).setAcceleration(servoNum, acceleration);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickServoActions class.");
        }
    }

    @RuleAction(label = "Get Acceleration")
    public @ActionOutput(name = "acceleration", type="int")
           Map<String, Object> getAcceleration(
            @ActionInput(name = "servoNum") short servoNum) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickServo)this.handler.getDevice()).getAcceleration(servoNum);
        result.put("acceleration", value);
        return result;
    }

    public static Map<String, Object> getAcceleration(@Nullable ThingActions actions, short servoNum) throws TinkerforgeException {
        if (actions instanceof BrickServoActions) {
            return ((BrickServoActions) actions).getAcceleration(servoNum);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickServoActions class.");
        }
    }

    @RuleAction(label = "Set Output Voltage")
    public void setOutputVoltage(
            @ActionInput(name = "voltage") int voltage) throws TinkerforgeException {
        ((BrickServo)this.handler.getDevice()).setOutputVoltage(voltage);
    }

    public static void setOutputVoltage(@Nullable ThingActions actions, int voltage) throws TinkerforgeException {
        if (actions instanceof BrickServoActions) {
            ((BrickServoActions) actions).setOutputVoltage(voltage);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickServoActions class.");
        }
    }

    @RuleAction(label = "Get Output Voltage")
    public @ActionOutput(name = "voltage", type="int")
           Map<String, Object> getOutputVoltage(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickServo)this.handler.getDevice()).getOutputVoltage();
        result.put("voltage", value);
        return result;
    }

    public static Map<String, Object> getOutputVoltage(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickServoActions) {
            return ((BrickServoActions) actions).getOutputVoltage();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickServoActions class.");
        }
    }

    @RuleAction(label = "Set Pulse Width")
    public void setPulseWidth(
            @ActionInput(name = "servoNum") short servoNum,
            @ActionInput(name = "min") int min,
            @ActionInput(name = "max") int max) throws TinkerforgeException {
        ((BrickServo)this.handler.getDevice()).setPulseWidth(servoNum, min, max);
    }

    public static void setPulseWidth(@Nullable ThingActions actions, short servoNum, int min, int max) throws TinkerforgeException {
        if (actions instanceof BrickServoActions) {
            ((BrickServoActions) actions).setPulseWidth(servoNum, min, max);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickServoActions class.");
        }
    }

    @RuleAction(label = "Get Pulse Width")
    public @ActionOutput(name = "min", type="int")
           @ActionOutput(name = "max", type="int")
           Map<String, Object> getPulseWidth(
            @ActionInput(name = "servoNum") short servoNum) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickServo.PulseWidth value = ((BrickServo)this.handler.getDevice()).getPulseWidth(servoNum);
        result.put("min", value.min);
        result.put("max", value.max);
        return result;
    }

    public static Map<String, Object> getPulseWidth(@Nullable ThingActions actions, short servoNum) throws TinkerforgeException {
        if (actions instanceof BrickServoActions) {
            return ((BrickServoActions) actions).getPulseWidth(servoNum);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickServoActions class.");
        }
    }

    @RuleAction(label = "Set Degree")
    public void setDegree(
            @ActionInput(name = "servoNum") short servoNum,
            @ActionInput(name = "min") short min,
            @ActionInput(name = "max") short max) throws TinkerforgeException {
        ((BrickServo)this.handler.getDevice()).setDegree(servoNum, min, max);
    }

    public static void setDegree(@Nullable ThingActions actions, short servoNum, short min, short max) throws TinkerforgeException {
        if (actions instanceof BrickServoActions) {
            ((BrickServoActions) actions).setDegree(servoNum, min, max);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickServoActions class.");
        }
    }

    @RuleAction(label = "Get Degree")
    public @ActionOutput(name = "min", type="short")
           @ActionOutput(name = "max", type="short")
           Map<String, Object> getDegree(
            @ActionInput(name = "servoNum") short servoNum) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickServo.Degree value = ((BrickServo)this.handler.getDevice()).getDegree(servoNum);
        result.put("min", value.min);
        result.put("max", value.max);
        return result;
    }

    public static Map<String, Object> getDegree(@Nullable ThingActions actions, short servoNum) throws TinkerforgeException {
        if (actions instanceof BrickServoActions) {
            return ((BrickServoActions) actions).getDegree(servoNum);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickServoActions class.");
        }
    }

    @RuleAction(label = "Set Period")
    public void setPeriod(
            @ActionInput(name = "servoNum") short servoNum,
            @ActionInput(name = "period") int period) throws TinkerforgeException {
        ((BrickServo)this.handler.getDevice()).setPeriod(servoNum, period);
    }

    public static void setPeriod(@Nullable ThingActions actions, short servoNum, int period) throws TinkerforgeException {
        if (actions instanceof BrickServoActions) {
            ((BrickServoActions) actions).setPeriod(servoNum, period);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickServoActions class.");
        }
    }

    @RuleAction(label = "Get Period")
    public @ActionOutput(name = "period", type="int")
           Map<String, Object> getPeriod(
            @ActionInput(name = "servoNum") short servoNum) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickServo)this.handler.getDevice()).getPeriod(servoNum);
        result.put("period", value);
        return result;
    }

    public static Map<String, Object> getPeriod(@Nullable ThingActions actions, short servoNum) throws TinkerforgeException {
        if (actions instanceof BrickServoActions) {
            return ((BrickServoActions) actions).getPeriod(servoNum);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickServoActions class.");
        }
    }

    @RuleAction(label = "Get Servo Current")
    public @ActionOutput(name = "current", type="int")
           Map<String, Object> getServoCurrent(
            @ActionInput(name = "servoNum") short servoNum) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickServo)this.handler.getDevice()).getServoCurrent(servoNum);
        result.put("current", value);
        return result;
    }

    public static Map<String, Object> getServoCurrent(@Nullable ThingActions actions, short servoNum) throws TinkerforgeException {
        if (actions instanceof BrickServoActions) {
            return ((BrickServoActions) actions).getServoCurrent(servoNum);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickServoActions class.");
        }
    }

    @RuleAction(label = "Get Overall Current")
    public @ActionOutput(name = "current", type="int")
           Map<String, Object> getOverallCurrent(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickServo)this.handler.getDevice()).getOverallCurrent();
        result.put("current", value);
        return result;
    }

    public static Map<String, Object> getOverallCurrent(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickServoActions) {
            return ((BrickServoActions) actions).getOverallCurrent();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickServoActions class.");
        }
    }

    @RuleAction(label = "Get Stack Input Voltage")
    public @ActionOutput(name = "voltage", type="int")
           Map<String, Object> getStackInputVoltage(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickServo)this.handler.getDevice()).getStackInputVoltage();
        result.put("voltage", value);
        return result;
    }

    public static Map<String, Object> getStackInputVoltage(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickServoActions) {
            return ((BrickServoActions) actions).getStackInputVoltage();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickServoActions class.");
        }
    }

    @RuleAction(label = "Get External Input Voltage")
    public @ActionOutput(name = "voltage", type="int")
           Map<String, Object> getExternalInputVoltage(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickServo)this.handler.getDevice()).getExternalInputVoltage();
        result.put("voltage", value);
        return result;
    }

    public static Map<String, Object> getExternalInputVoltage(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickServoActions) {
            return ((BrickServoActions) actions).getExternalInputVoltage();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickServoActions class.");
        }
    }

    @RuleAction(label = "Get Minimum Voltage")
    public @ActionOutput(name = "voltage", type="int")
           Map<String, Object> getMinimumVoltage(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickServo)this.handler.getDevice()).getMinimumVoltage();
        result.put("voltage", value);
        return result;
    }

    public static Map<String, Object> getMinimumVoltage(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickServoActions) {
            return ((BrickServoActions) actions).getMinimumVoltage();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickServoActions class.");
        }
    }
}
