package com.tinkerforge;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletAnalogOutV2Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Output Voltage")
    public @ActionOutput(name = "voltage", type="int")
           Map<String, Object> getOutputVoltage(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletAnalogOutV2)this.handler.getDevice()).getOutputVoltage();
        result.put("voltage", value);
        return result;
    }

    public static Map<String, Object> getOutputVoltage(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletAnalogOutV2Actions) {
            return ((BrickletAnalogOutV2Actions) actions).getOutputVoltage();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAnalogOutV2Actions class.");
        }
    }

    @RuleAction(label = "Get Input Voltage")
    public @ActionOutput(name = "voltage", type="int")
           Map<String, Object> getInputVoltage(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletAnalogOutV2)this.handler.getDevice()).getInputVoltage();
        result.put("voltage", value);
        return result;
    }

    public static Map<String, Object> getInputVoltage(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletAnalogOutV2Actions) {
            return ((BrickletAnalogOutV2Actions) actions).getInputVoltage();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletAnalogOutV2Actions class.");
        }
    }
}
