package com.tinkerforge;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletCompassActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Heading")
    public @ActionOutput(name = "heading", type="int")
           Map<String, Object> getHeading(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletCompass)this.handler.getDevice()).getHeading();
        result.put("heading", value);
        return result;
    }

    public static Map<String, Object> getHeading(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletCompassActions) {
            return ((BrickletCompassActions) actions).getHeading();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletCompassActions class.");
        }
    }

    @RuleAction(label = "Get Magnetic Flux Density")
    public @ActionOutput(name = "x", type="int")
           @ActionOutput(name = "y", type="int")
           @ActionOutput(name = "z", type="int")
           Map<String, Object> getMagneticFluxDensity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletCompass.MagneticFluxDensity value = ((BrickletCompass)this.handler.getDevice()).getMagneticFluxDensity();
        result.put("x", value.x);
        result.put("y", value.y);
        result.put("z", value.z);
        return result;
    }

    public static Map<String, Object> getMagneticFluxDensity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletCompassActions) {
            return ((BrickletCompassActions) actions).getMagneticFluxDensity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletCompassActions class.");
        }
    }

    @RuleAction(label = "Get Configuration")
    public @ActionOutput(name = "dataRate", type="int")
           @ActionOutput(name = "backgroundCalibration", type="boolean")
           Map<String, Object> getConfiguration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletCompass.Configuration value = ((BrickletCompass)this.handler.getDevice()).getConfiguration();
        result.put("dataRate", value.dataRate);
        result.put("backgroundCalibration", value.backgroundCalibration);
        return result;
    }

    public static Map<String, Object> getConfiguration(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletCompassActions) {
            return ((BrickletCompassActions) actions).getConfiguration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletCompassActions class.");
        }
    }

    @RuleAction(label = "Get Calibration")
    public @ActionOutput(name = "offset", type="int[]")
           @ActionOutput(name = "gain", type="int[]")
           Map<String, Object> getCalibration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletCompass.Calibration value = ((BrickletCompass)this.handler.getDevice()).getCalibration();
        result.put("offset", value.offset);
        result.put("gain", value.gain);
        return result;
    }

    public static Map<String, Object> getCalibration(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletCompassActions) {
            return ((BrickletCompassActions) actions).getCalibration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletCompassActions class.");
        }
    }
}
