package com.tinkerforge;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletDistanceIRV2Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Distance")
    public @ActionOutput(name = "distance", type="int")
           Map<String, Object> getDistance(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletDistanceIRV2)this.handler.getDevice()).getDistance();
        result.put("distance", value);
        return result;
    }

    public static Map<String, Object> getDistance(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletDistanceIRV2Actions) {
            return ((BrickletDistanceIRV2Actions) actions).getDistance();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletDistanceIRV2Actions class.");
        }
    }

    @RuleAction(label = "Get Analog Value")
    public @ActionOutput(name = "analogValue", type="long")
           Map<String, Object> getAnalogValue(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        long value = ((BrickletDistanceIRV2)this.handler.getDevice()).getAnalogValue();
        result.put("analogValue", value);
        return result;
    }

    public static Map<String, Object> getAnalogValue(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletDistanceIRV2Actions) {
            return ((BrickletDistanceIRV2Actions) actions).getAnalogValue();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletDistanceIRV2Actions class.");
        }
    }

    @RuleAction(label = "Get Moving Average Configuration")
    public @ActionOutput(name = "movingAverageLength", type="int")
           Map<String, Object> getMovingAverageConfiguration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletDistanceIRV2)this.handler.getDevice()).getMovingAverageConfiguration();
        result.put("movingAverageLength", value);
        return result;
    }

    public static Map<String, Object> getMovingAverageConfiguration(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletDistanceIRV2Actions) {
            return ((BrickletDistanceIRV2Actions) actions).getMovingAverageConfiguration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletDistanceIRV2Actions class.");
        }
    }

    @RuleAction(label = "Get Distance LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> getDistanceLEDConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletDistanceIRV2)this.handler.getDevice()).getDistanceLEDConfig();
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> getDistanceLEDConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletDistanceIRV2Actions) {
            return ((BrickletDistanceIRV2Actions) actions).getDistanceLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletDistanceIRV2Actions class.");
        }
    }

    @RuleAction(label = "Get Sensor Type")
    public @ActionOutput(name = "sensor", type="int")
           Map<String, Object> getSensorType(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletDistanceIRV2)this.handler.getDevice()).getSensorType();
        result.put("sensor", value);
        return result;
    }

    public static Map<String, Object> getSensorType(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletDistanceIRV2Actions) {
            return ((BrickletDistanceIRV2Actions) actions).getSensorType();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletDistanceIRV2Actions class.");
        }
    }
}
