package com.tinkerforge;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletDualButtonV2Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get LED State")
    public @ActionOutput(name = "ledL", type="int")
           @ActionOutput(name = "ledR", type="int")
           Map<String, Object> getLEDState(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletDualButtonV2.LEDState value = ((BrickletDualButtonV2)this.handler.getDevice()).getLEDState();
        result.put("ledL", value.ledL);
        result.put("ledR", value.ledR);
        return result;
    }

    public static Map<String, Object> getLEDState(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletDualButtonV2Actions) {
            return ((BrickletDualButtonV2Actions) actions).getLEDState();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletDualButtonV2Actions class.");
        }
    }

    @RuleAction(label = "Get Button State")
    public @ActionOutput(name = "buttonL", type="int")
           @ActionOutput(name = "buttonR", type="int")
           Map<String, Object> getButtonState(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletDualButtonV2.ButtonState value = ((BrickletDualButtonV2)this.handler.getDevice()).getButtonState();
        result.put("buttonL", value.buttonL);
        result.put("buttonR", value.buttonR);
        return result;
    }

    public static Map<String, Object> getButtonState(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletDualButtonV2Actions) {
            return ((BrickletDualButtonV2Actions) actions).getButtonState();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletDualButtonV2Actions class.");
        }
    }
}
