package com.tinkerforge;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletHallEffectV2Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Magnetic Flux Density")
    public @ActionOutput(name = "magneticFluxDensity", type="int")
           Map<String, Object> getMagneticFluxDensity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletHallEffectV2)this.handler.getDevice()).getMagneticFluxDensity();
        result.put("magneticFluxDensity", value);
        return result;
    }

    public static Map<String, Object> getMagneticFluxDensity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletHallEffectV2Actions) {
            return ((BrickletHallEffectV2Actions) actions).getMagneticFluxDensity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletHallEffectV2Actions class.");
        }
    }

    @RuleAction(label = "Get Counter")
    public @ActionOutput(name = "count", type="long")
           Map<String, Object> getCounter(
            @ActionInput(name = "resetCounter") boolean resetCounter) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        long value = ((BrickletHallEffectV2)this.handler.getDevice()).getCounter(resetCounter);
        result.put("count", value);
        return result;
    }

    public static Map<String, Object> getCounter(@Nullable ThingActions actions, boolean resetCounter) throws TinkerforgeException {
        if (actions instanceof BrickletHallEffectV2Actions) {
            return ((BrickletHallEffectV2Actions) actions).getCounter(resetCounter);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletHallEffectV2Actions class.");
        }
    }

    @RuleAction(label = "Get Counter Config")
    public @ActionOutput(name = "highThreshold", type="int")
           @ActionOutput(name = "lowThreshold", type="int")
           @ActionOutput(name = "debounce", type="long")
           Map<String, Object> getCounterConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletHallEffectV2.CounterConfig value = ((BrickletHallEffectV2)this.handler.getDevice()).getCounterConfig();
        result.put("highThreshold", value.highThreshold);
        result.put("lowThreshold", value.lowThreshold);
        result.put("debounce", value.debounce);
        return result;
    }

    public static Map<String, Object> getCounterConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletHallEffectV2Actions) {
            return ((BrickletHallEffectV2Actions) actions).getCounterConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletHallEffectV2Actions class.");
        }
    }
}
