package com.tinkerforge;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletIO16Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Port")
    public @ActionOutput(name = "valueMask", type="short")
           Map<String, Object> getPort(
            @ActionInput(name = "port") char port) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        short value = ((BrickletIO16)this.handler.getDevice()).getPort(port);
        result.put("valueMask", value);
        return result;
    }

    public static Map<String, Object> getPort(@Nullable ThingActions actions, char port) throws TinkerforgeException {
        if (actions instanceof BrickletIO16Actions) {
            return ((BrickletIO16Actions) actions).getPort(port);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIO16Actions class.");
        }
    }

    @RuleAction(label = "Get Port Configuration")
    public @ActionOutput(name = "directionMask", type="short")
           @ActionOutput(name = "valueMask", type="short")
           Map<String, Object> getPortConfiguration(
            @ActionInput(name = "port") char port) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletIO16.PortConfiguration value = ((BrickletIO16)this.handler.getDevice()).getPortConfiguration(port);
        result.put("directionMask", value.directionMask);
        result.put("valueMask", value.valueMask);
        return result;
    }

    public static Map<String, Object> getPortConfiguration(@Nullable ThingActions actions, char port) throws TinkerforgeException {
        if (actions instanceof BrickletIO16Actions) {
            return ((BrickletIO16Actions) actions).getPortConfiguration(port);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIO16Actions class.");
        }
    }

    @RuleAction(label = "Get Edge Count")
    public @ActionOutput(name = "count", type="long")
           Map<String, Object> getEdgeCount(
            @ActionInput(name = "pin") short pin,
            @ActionInput(name = "resetCounter") boolean resetCounter) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        long value = ((BrickletIO16)this.handler.getDevice()).getEdgeCount(pin, resetCounter);
        result.put("count", value);
        return result;
    }

    public static Map<String, Object> getEdgeCount(@Nullable ThingActions actions, short pin, boolean resetCounter) throws TinkerforgeException {
        if (actions instanceof BrickletIO16Actions) {
            return ((BrickletIO16Actions) actions).getEdgeCount(pin, resetCounter);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIO16Actions class.");
        }
    }

    @RuleAction(label = "Get Port Monoflop")
    public @ActionOutput(name = "value", type="short")
           @ActionOutput(name = "time", type="long")
           @ActionOutput(name = "timeRemaining", type="long")
           Map<String, Object> getPortMonoflop(
            @ActionInput(name = "port") char port,
            @ActionInput(name = "pin") short pin) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletIO16.PortMonoflop value = ((BrickletIO16)this.handler.getDevice()).getPortMonoflop(port, pin);
        result.put("value", value.value);
        result.put("time", value.time);
        result.put("timeRemaining", value.timeRemaining);
        return result;
    }

    public static Map<String, Object> getPortMonoflop(@Nullable ThingActions actions, char port, short pin) throws TinkerforgeException {
        if (actions instanceof BrickletIO16Actions) {
            return ((BrickletIO16Actions) actions).getPortMonoflop(port, pin);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIO16Actions class.");
        }
    }

    @RuleAction(label = "Get Edge Count Config")
    public @ActionOutput(name = "edgeType", type="short")
           @ActionOutput(name = "debounce", type="short")
           Map<String, Object> getEdgeCountConfig(
            @ActionInput(name = "pin") short pin) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletIO16.EdgeCountConfig value = ((BrickletIO16)this.handler.getDevice()).getEdgeCountConfig(pin);
        result.put("edgeType", value.edgeType);
        result.put("debounce", value.debounce);
        return result;
    }

    public static Map<String, Object> getEdgeCountConfig(@Nullable ThingActions actions, short pin) throws TinkerforgeException {
        if (actions instanceof BrickletIO16Actions) {
            return ((BrickletIO16Actions) actions).getEdgeCountConfig(pin);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIO16Actions class.");
        }
    }
}
