package com.tinkerforge;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletIO4Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Value")
    public @ActionOutput(name = "valueMask", type="short")
           Map<String, Object> getValue(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        short value = ((BrickletIO4)this.handler.getDevice()).getValue();
        result.put("valueMask", value);
        return result;
    }

    public static Map<String, Object> getValue(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletIO4Actions) {
            return ((BrickletIO4Actions) actions).getValue();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIO4Actions class.");
        }
    }

    @RuleAction(label = "Get Configuration")
    public @ActionOutput(name = "directionMask", type="short")
           @ActionOutput(name = "valueMask", type="short")
           Map<String, Object> getConfiguration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletIO4.Configuration value = ((BrickletIO4)this.handler.getDevice()).getConfiguration();
        result.put("directionMask", value.directionMask);
        result.put("valueMask", value.valueMask);
        return result;
    }

    public static Map<String, Object> getConfiguration(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletIO4Actions) {
            return ((BrickletIO4Actions) actions).getConfiguration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIO4Actions class.");
        }
    }

    @RuleAction(label = "Get Edge Count")
    public @ActionOutput(name = "count", type="long")
           Map<String, Object> getEdgeCount(
            @ActionInput(name = "pin") short pin,
            @ActionInput(name = "resetCounter") boolean resetCounter) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        long value = ((BrickletIO4)this.handler.getDevice()).getEdgeCount(pin, resetCounter);
        result.put("count", value);
        return result;
    }

    public static Map<String, Object> getEdgeCount(@Nullable ThingActions actions, short pin, boolean resetCounter) throws TinkerforgeException {
        if (actions instanceof BrickletIO4Actions) {
            return ((BrickletIO4Actions) actions).getEdgeCount(pin, resetCounter);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIO4Actions class.");
        }
    }

    @RuleAction(label = "Get Monoflop")
    public @ActionOutput(name = "value", type="short")
           @ActionOutput(name = "time", type="long")
           @ActionOutput(name = "timeRemaining", type="long")
           Map<String, Object> getMonoflop(
            @ActionInput(name = "pin") short pin) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletIO4.Monoflop value = ((BrickletIO4)this.handler.getDevice()).getMonoflop(pin);
        result.put("value", value.value);
        result.put("time", value.time);
        result.put("timeRemaining", value.timeRemaining);
        return result;
    }

    public static Map<String, Object> getMonoflop(@Nullable ThingActions actions, short pin) throws TinkerforgeException {
        if (actions instanceof BrickletIO4Actions) {
            return ((BrickletIO4Actions) actions).getMonoflop(pin);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIO4Actions class.");
        }
    }

    @RuleAction(label = "Get Edge Count Config")
    public @ActionOutput(name = "edgeType", type="short")
           @ActionOutput(name = "debounce", type="short")
           Map<String, Object> getEdgeCountConfig(
            @ActionInput(name = "pin") short pin) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletIO4.EdgeCountConfig value = ((BrickletIO4)this.handler.getDevice()).getEdgeCountConfig(pin);
        result.put("edgeType", value.edgeType);
        result.put("debounce", value.debounce);
        return result;
    }

    public static Map<String, Object> getEdgeCountConfig(@Nullable ThingActions actions, short pin) throws TinkerforgeException {
        if (actions instanceof BrickletIO4Actions) {
            return ((BrickletIO4Actions) actions).getEdgeCountConfig(pin);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIO4Actions class.");
        }
    }
}
