package com.tinkerforge;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletIO4V2Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Value")
    public @ActionOutput(name = "value", type="boolean[]")
           Map<String, Object> getValue(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        boolean[] value = ((BrickletIO4V2)this.handler.getDevice()).getValue();
        result.put("value", value);
        return result;
    }

    public static Map<String, Object> getValue(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletIO4V2Actions) {
            return ((BrickletIO4V2Actions) actions).getValue();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIO4V2Actions class.");
        }
    }

    @RuleAction(label = "Get Configuration")
    public @ActionOutput(name = "direction", type="char")
           @ActionOutput(name = "value", type="boolean")
           Map<String, Object> getConfiguration(
            @ActionInput(name = "channel") int channel) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletIO4V2.Configuration value = ((BrickletIO4V2)this.handler.getDevice()).getConfiguration(channel);
        result.put("direction", value.direction);
        result.put("value", value.value);
        return result;
    }

    public static Map<String, Object> getConfiguration(@Nullable ThingActions actions, int channel) throws TinkerforgeException {
        if (actions instanceof BrickletIO4V2Actions) {
            return ((BrickletIO4V2Actions) actions).getConfiguration(channel);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIO4V2Actions class.");
        }
    }

    @RuleAction(label = "Get Edge Count")
    public @ActionOutput(name = "count", type="long")
           Map<String, Object> getEdgeCount(
            @ActionInput(name = "channel") int channel,
            @ActionInput(name = "resetCounter") boolean resetCounter) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        long value = ((BrickletIO4V2)this.handler.getDevice()).getEdgeCount(channel, resetCounter);
        result.put("count", value);
        return result;
    }

    public static Map<String, Object> getEdgeCount(@Nullable ThingActions actions, int channel, boolean resetCounter) throws TinkerforgeException {
        if (actions instanceof BrickletIO4V2Actions) {
            return ((BrickletIO4V2Actions) actions).getEdgeCount(channel, resetCounter);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIO4V2Actions class.");
        }
    }

    @RuleAction(label = "Get Monoflop")
    public @ActionOutput(name = "value", type="boolean")
           @ActionOutput(name = "time", type="long")
           @ActionOutput(name = "timeRemaining", type="long")
           Map<String, Object> getMonoflop(
            @ActionInput(name = "channel") int channel) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletIO4V2.Monoflop value = ((BrickletIO4V2)this.handler.getDevice()).getMonoflop(channel);
        result.put("value", value.value);
        result.put("time", value.time);
        result.put("timeRemaining", value.timeRemaining);
        return result;
    }

    public static Map<String, Object> getMonoflop(@Nullable ThingActions actions, int channel) throws TinkerforgeException {
        if (actions instanceof BrickletIO4V2Actions) {
            return ((BrickletIO4V2Actions) actions).getMonoflop(channel);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIO4V2Actions class.");
        }
    }

    @RuleAction(label = "Get Edge Count Configuration")
    public @ActionOutput(name = "edgeType", type="int")
           @ActionOutput(name = "debounce", type="int")
           Map<String, Object> getEdgeCountConfiguration(
            @ActionInput(name = "channel") int channel) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletIO4V2.EdgeCountConfiguration value = ((BrickletIO4V2)this.handler.getDevice()).getEdgeCountConfiguration(channel);
        result.put("edgeType", value.edgeType);
        result.put("debounce", value.debounce);
        return result;
    }

    public static Map<String, Object> getEdgeCountConfiguration(@Nullable ThingActions actions, int channel) throws TinkerforgeException {
        if (actions instanceof BrickletIO4V2Actions) {
            return ((BrickletIO4V2Actions) actions).getEdgeCountConfiguration(channel);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIO4V2Actions class.");
        }
    }

    @RuleAction(label = "Set PWM Configuration")
    public void setPWMConfiguration(
            @ActionInput(name = "channel") int channel,
            @ActionInput(name = "frequency") long frequency,
            @ActionInput(name = "dutyCycle") int dutyCycle) throws TinkerforgeException {
        ((BrickletIO4V2)this.handler.getDevice()).setPWMConfiguration(channel, frequency, dutyCycle);
    }

    public static void setPWMConfiguration(@Nullable ThingActions actions, int channel, long frequency, int dutyCycle) throws TinkerforgeException {
        if (actions instanceof BrickletIO4V2Actions) {
            ((BrickletIO4V2Actions) actions).setPWMConfiguration(channel, frequency, dutyCycle);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIO4V2Actions class.");
        }
    }

    @RuleAction(label = "Get PWM Configuration")
    public @ActionOutput(name = "frequency", type="long")
           @ActionOutput(name = "dutyCycle", type="int")
           Map<String, Object> getPWMConfiguration(
            @ActionInput(name = "channel") int channel) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletIO4V2.PWMConfiguration value = ((BrickletIO4V2)this.handler.getDevice()).getPWMConfiguration(channel);
        result.put("frequency", value.frequency);
        result.put("dutyCycle", value.dutyCycle);
        return result;
    }

    public static Map<String, Object> getPWMConfiguration(@Nullable ThingActions actions, int channel) throws TinkerforgeException {
        if (actions instanceof BrickletIO4V2Actions) {
            return ((BrickletIO4V2Actions) actions).getPWMConfiguration(channel);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIO4V2Actions class.");
        }
    }
}
