package com.tinkerforge;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletIndustrialAnalogOutActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Is Enabled")
    public @ActionOutput(name = "enabled", type="boolean")
           Map<String, Object> isEnabled(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        boolean value = ((BrickletIndustrialAnalogOut)this.handler.getDevice()).isEnabled();
        result.put("enabled", value);
        return result;
    }

    public static Map<String, Object> isEnabled(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialAnalogOutActions) {
            return ((BrickletIndustrialAnalogOutActions) actions).isEnabled();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialAnalogOutActions class.");
        }
    }

    @RuleAction(label = "Get Voltage")
    public @ActionOutput(name = "voltage", type="int")
           Map<String, Object> getVoltage(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletIndustrialAnalogOut)this.handler.getDevice()).getVoltage();
        result.put("voltage", value);
        return result;
    }

    public static Map<String, Object> getVoltage(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialAnalogOutActions) {
            return ((BrickletIndustrialAnalogOutActions) actions).getVoltage();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialAnalogOutActions class.");
        }
    }

    @RuleAction(label = "Get Current")
    public @ActionOutput(name = "current", type="int")
           Map<String, Object> getCurrent(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletIndustrialAnalogOut)this.handler.getDevice()).getCurrent();
        result.put("current", value);
        return result;
    }

    public static Map<String, Object> getCurrent(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialAnalogOutActions) {
            return ((BrickletIndustrialAnalogOutActions) actions).getCurrent();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialAnalogOutActions class.");
        }
    }

    @RuleAction(label = "Get Configuration")
    public @ActionOutput(name = "voltageRange", type="short")
           @ActionOutput(name = "currentRange", type="short")
           Map<String, Object> getConfiguration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletIndustrialAnalogOut.Configuration value = ((BrickletIndustrialAnalogOut)this.handler.getDevice()).getConfiguration();
        result.put("voltageRange", value.voltageRange);
        result.put("currentRange", value.currentRange);
        return result;
    }

    public static Map<String, Object> getConfiguration(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialAnalogOutActions) {
            return ((BrickletIndustrialAnalogOutActions) actions).getConfiguration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialAnalogOutActions class.");
        }
    }
}
