package com.tinkerforge;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletIndustrialDigitalIn4V2Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Value")
    public @ActionOutput(name = "value", type="boolean[]")
           Map<String, Object> getValue(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        boolean[] value = ((BrickletIndustrialDigitalIn4V2)this.handler.getDevice()).getValue();
        result.put("value", value);
        return result;
    }

    public static Map<String, Object> getValue(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialDigitalIn4V2Actions) {
            return ((BrickletIndustrialDigitalIn4V2Actions) actions).getValue();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialDigitalIn4V2Actions class.");
        }
    }

    @RuleAction(label = "Get Channel LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> getChannelLEDConfig(
            @ActionInput(name = "channel") int channel) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletIndustrialDigitalIn4V2)this.handler.getDevice()).getChannelLEDConfig(channel);
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> getChannelLEDConfig(@Nullable ThingActions actions, int channel) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialDigitalIn4V2Actions) {
            return ((BrickletIndustrialDigitalIn4V2Actions) actions).getChannelLEDConfig(channel);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialDigitalIn4V2Actions class.");
        }
    }

    @RuleAction(label = "Get Edge Count Configuration")
    public @ActionOutput(name = "edgeType", type="int")
           @ActionOutput(name = "debounce", type="int")
           Map<String, Object> getEdgeCountConfiguration(
            @ActionInput(name = "channel") int channel) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletIndustrialDigitalIn4V2.EdgeCountConfiguration value = ((BrickletIndustrialDigitalIn4V2)this.handler.getDevice()).getEdgeCountConfiguration(channel);
        result.put("edgeType", value.edgeType);
        result.put("debounce", value.debounce);
        return result;
    }

    public static Map<String, Object> getEdgeCountConfiguration(@Nullable ThingActions actions, int channel) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialDigitalIn4V2Actions) {
            return ((BrickletIndustrialDigitalIn4V2Actions) actions).getEdgeCountConfiguration(channel);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialDigitalIn4V2Actions class.");
        }
    }
}
