package com.tinkerforge;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletIndustrialDualAnalogInActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Voltage")
    public @ActionOutput(name = "voltage", type="int")
           Map<String, Object> getVoltage(
            @ActionInput(name = "channel") short channel) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletIndustrialDualAnalogIn)this.handler.getDevice()).getVoltage(channel);
        result.put("voltage", value);
        return result;
    }

    public static Map<String, Object> getVoltage(@Nullable ThingActions actions, short channel) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialDualAnalogInActions) {
            return ((BrickletIndustrialDualAnalogInActions) actions).getVoltage(channel);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialDualAnalogInActions class.");
        }
    }

    @RuleAction(label = "Get Sample Rate")
    public @ActionOutput(name = "rate", type="short")
           Map<String, Object> getSampleRate(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        short value = ((BrickletIndustrialDualAnalogIn)this.handler.getDevice()).getSampleRate();
        result.put("rate", value);
        return result;
    }

    public static Map<String, Object> getSampleRate(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialDualAnalogInActions) {
            return ((BrickletIndustrialDualAnalogInActions) actions).getSampleRate();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialDualAnalogInActions class.");
        }
    }

    @RuleAction(label = "Get Calibration")
    public @ActionOutput(name = "offset", type="int[]")
           @ActionOutput(name = "gain", type="int[]")
           Map<String, Object> getCalibration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletIndustrialDualAnalogIn.Calibration value = ((BrickletIndustrialDualAnalogIn)this.handler.getDevice()).getCalibration();
        result.put("offset", value.offset);
        result.put("gain", value.gain);
        return result;
    }

    public static Map<String, Object> getCalibration(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialDualAnalogInActions) {
            return ((BrickletIndustrialDualAnalogInActions) actions).getCalibration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialDualAnalogInActions class.");
        }
    }

    @RuleAction(label = "Get ADC Values")
    public @ActionOutput(name = "value", type="int[]")
           Map<String, Object> getADCValues(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int[] value = ((BrickletIndustrialDualAnalogIn)this.handler.getDevice()).getADCValues();
        result.put("value", value);
        return result;
    }

    public static Map<String, Object> getADCValues(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialDualAnalogInActions) {
            return ((BrickletIndustrialDualAnalogInActions) actions).getADCValues();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialDualAnalogInActions class.");
        }
    }
}
