package com.tinkerforge;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletIndustrialDualAnalogInV2Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Voltage")
    public @ActionOutput(name = "voltage", type="int")
           Map<String, Object> getVoltage(
            @ActionInput(name = "channel") int channel) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletIndustrialDualAnalogInV2)this.handler.getDevice()).getVoltage(channel);
        result.put("voltage", value);
        return result;
    }

    public static Map<String, Object> getVoltage(@Nullable ThingActions actions, int channel) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialDualAnalogInV2Actions) {
            return ((BrickletIndustrialDualAnalogInV2Actions) actions).getVoltage(channel);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialDualAnalogInV2Actions class.");
        }
    }

    @RuleAction(label = "Get Channel LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> getChannelLEDConfig(
            @ActionInput(name = "channel") int channel) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletIndustrialDualAnalogInV2)this.handler.getDevice()).getChannelLEDConfig(channel);
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> getChannelLEDConfig(@Nullable ThingActions actions, int channel) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialDualAnalogInV2Actions) {
            return ((BrickletIndustrialDualAnalogInV2Actions) actions).getChannelLEDConfig(channel);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialDualAnalogInV2Actions class.");
        }
    }

    @RuleAction(label = "Get Channel LED Status Config")
    public @ActionOutput(name = "min", type="int")
           @ActionOutput(name = "max", type="int")
           @ActionOutput(name = "config", type="int")
           Map<String, Object> getChannelLEDStatusConfig(
            @ActionInput(name = "channel") int channel) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletIndustrialDualAnalogInV2.ChannelLEDStatusConfig value = ((BrickletIndustrialDualAnalogInV2)this.handler.getDevice()).getChannelLEDStatusConfig(channel);
        result.put("min", value.min);
        result.put("max", value.max);
        result.put("config", value.config);
        return result;
    }

    public static Map<String, Object> getChannelLEDStatusConfig(@Nullable ThingActions actions, int channel) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialDualAnalogInV2Actions) {
            return ((BrickletIndustrialDualAnalogInV2Actions) actions).getChannelLEDStatusConfig(channel);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialDualAnalogInV2Actions class.");
        }
    }

    @RuleAction(label = "Get Sample Rate")
    public @ActionOutput(name = "rate", type="int")
           Map<String, Object> getSampleRate(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletIndustrialDualAnalogInV2)this.handler.getDevice()).getSampleRate();
        result.put("rate", value);
        return result;
    }

    public static Map<String, Object> getSampleRate(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialDualAnalogInV2Actions) {
            return ((BrickletIndustrialDualAnalogInV2Actions) actions).getSampleRate();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialDualAnalogInV2Actions class.");
        }
    }

    @RuleAction(label = "Get Calibration")
    public @ActionOutput(name = "offset", type="int[]")
           @ActionOutput(name = "gain", type="int[]")
           Map<String, Object> getCalibration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletIndustrialDualAnalogInV2.Calibration value = ((BrickletIndustrialDualAnalogInV2)this.handler.getDevice()).getCalibration();
        result.put("offset", value.offset);
        result.put("gain", value.gain);
        return result;
    }

    public static Map<String, Object> getCalibration(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialDualAnalogInV2Actions) {
            return ((BrickletIndustrialDualAnalogInV2Actions) actions).getCalibration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialDualAnalogInV2Actions class.");
        }
    }

    @RuleAction(label = "Get ADC Values")
    public @ActionOutput(name = "value", type="int[]")
           Map<String, Object> getADCValues(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int[] value = ((BrickletIndustrialDualAnalogInV2)this.handler.getDevice()).getADCValues();
        result.put("value", value);
        return result;
    }

    public static Map<String, Object> getADCValues(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialDualAnalogInV2Actions) {
            return ((BrickletIndustrialDualAnalogInV2Actions) actions).getADCValues();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialDualAnalogInV2Actions class.");
        }
    }
}
