package com.tinkerforge;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletIndustrialDualRelayActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Value")
    public @ActionOutput(name = "channel0", type="boolean")
           @ActionOutput(name = "channel1", type="boolean")
           Map<String, Object> getValue(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletIndustrialDualRelay.Value value = ((BrickletIndustrialDualRelay)this.handler.getDevice()).getValue();
        result.put("channel0", value.channel0);
        result.put("channel1", value.channel1);
        return result;
    }

    public static Map<String, Object> getValue(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialDualRelayActions) {
            return ((BrickletIndustrialDualRelayActions) actions).getValue();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialDualRelayActions class.");
        }
    }

    @RuleAction(label = "Get Monoflop")
    public @ActionOutput(name = "value", type="boolean")
           @ActionOutput(name = "time", type="long")
           @ActionOutput(name = "timeRemaining", type="long")
           Map<String, Object> getMonoflop(
            @ActionInput(name = "channel") int channel) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletIndustrialDualRelay.Monoflop value = ((BrickletIndustrialDualRelay)this.handler.getDevice()).getMonoflop(channel);
        result.put("value", value.value);
        result.put("time", value.time);
        result.put("timeRemaining", value.timeRemaining);
        return result;
    }

    public static Map<String, Object> getMonoflop(@Nullable ThingActions actions, int channel) throws TinkerforgeException {
        if (actions instanceof BrickletIndustrialDualRelayActions) {
            return ((BrickletIndustrialDualRelayActions) actions).getMonoflop(channel);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIndustrialDualRelayActions class.");
        }
    }
}
