package com.tinkerforge;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletIsolatorActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Statistics")
    public @ActionOutput(name = "messagesFromBrick", type="long")
           @ActionOutput(name = "messagesFromBricklet", type="long")
           @ActionOutput(name = "connectedBrickletDeviceIdentifier", type="int")
           @ActionOutput(name = "connectedBrickletUID", type="String")
           Map<String, Object> getStatistics(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletIsolator.Statistics value = ((BrickletIsolator)this.handler.getDevice()).getStatistics();
        result.put("messagesFromBrick", value.messagesFromBrick);
        result.put("messagesFromBricklet", value.messagesFromBricklet);
        result.put("connectedBrickletDeviceIdentifier", value.connectedBrickletDeviceIdentifier);
        result.put("connectedBrickletUID", value.connectedBrickletUID);
        return result;
    }

    public static Map<String, Object> getStatistics(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletIsolatorActions) {
            return ((BrickletIsolatorActions) actions).getStatistics();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletIsolatorActions class.");
        }
    }
}
