package com.tinkerforge;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletLCD128x64Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Write Pixels")
    public void writePixels(
            @ActionInput(name = "xStart") int xStart,
            @ActionInput(name = "yStart") int yStart,
            @ActionInput(name = "xEnd") int xEnd,
            @ActionInput(name = "yEnd") int yEnd,
            @ActionInput(name = "pixels") boolean[] pixels) throws TinkerforgeException {
        ((BrickletLCD128x64)this.handler.getDevice()).writePixels(xStart, yStart, xEnd, yEnd, pixels);
    }

    public static void writePixels(@Nullable ThingActions actions, int xStart, int yStart, int xEnd, int yEnd, boolean[] pixels) throws TinkerforgeException {
        if (actions instanceof BrickletLCD128x64Actions) {
            ((BrickletLCD128x64Actions) actions).writePixels(xStart, yStart, xEnd, yEnd, pixels);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLCD128x64Actions class.");
        }
    }

    @RuleAction(label = "Read Pixels")
    public @ActionOutput(name = "pixels", type="boolean[]")
           Map<String, Object> readPixels(
            @ActionInput(name = "xStart") int xStart,
            @ActionInput(name = "yStart") int yStart,
            @ActionInput(name = "xEnd") int xEnd,
            @ActionInput(name = "yEnd") int yEnd) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        boolean[] value = ((BrickletLCD128x64)this.handler.getDevice()).readPixels(xStart, yStart, xEnd, yEnd);
        result.put("pixels", value);
        return result;
    }

    public static Map<String, Object> readPixels(@Nullable ThingActions actions, int xStart, int yStart, int xEnd, int yEnd) throws TinkerforgeException {
        if (actions instanceof BrickletLCD128x64Actions) {
            return ((BrickletLCD128x64Actions) actions).readPixels(xStart, yStart, xEnd, yEnd);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLCD128x64Actions class.");
        }
    }

    @RuleAction(label = "Clear Display")
    public void clearDisplay(
            ) throws TinkerforgeException {
        ((BrickletLCD128x64)this.handler.getDevice()).clearDisplay();
    }

    public static void clearDisplay(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletLCD128x64Actions) {
            ((BrickletLCD128x64Actions) actions).clearDisplay();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLCD128x64Actions class.");
        }
    }

    @RuleAction(label = "Write Line")
    public void writeLine(
            @ActionInput(name = "line") int line,
            @ActionInput(name = "position") int position,
            @ActionInput(name = "text") String text) throws TinkerforgeException {
        ((BrickletLCD128x64)this.handler.getDevice()).writeLine(line, position, text);
    }

    public static void writeLine(@Nullable ThingActions actions, int line, int position, String text) throws TinkerforgeException {
        if (actions instanceof BrickletLCD128x64Actions) {
            ((BrickletLCD128x64Actions) actions).writeLine(line, position, text);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLCD128x64Actions class.");
        }
    }

    @RuleAction(label = "Draw Buffered Frame")
    public void drawBufferedFrame(
            @ActionInput(name = "forceCompleteRedraw") boolean forceCompleteRedraw) throws TinkerforgeException {
        ((BrickletLCD128x64)this.handler.getDevice()).drawBufferedFrame(forceCompleteRedraw);
    }

    public static void drawBufferedFrame(@Nullable ThingActions actions, boolean forceCompleteRedraw) throws TinkerforgeException {
        if (actions instanceof BrickletLCD128x64Actions) {
            ((BrickletLCD128x64Actions) actions).drawBufferedFrame(forceCompleteRedraw);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLCD128x64Actions class.");
        }
    }

    @RuleAction(label = "Get Touch Position")
    public @ActionOutput(name = "pressure", type="int")
           @ActionOutput(name = "x", type="int")
           @ActionOutput(name = "y", type="int")
           @ActionOutput(name = "age", type="long")
           Map<String, Object> getTouchPosition(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletLCD128x64.TouchPosition value = ((BrickletLCD128x64)this.handler.getDevice()).getTouchPosition();
        result.put("pressure", value.pressure);
        result.put("x", value.x);
        result.put("y", value.y);
        result.put("age", value.age);
        return result;
    }

    public static Map<String, Object> getTouchPosition(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletLCD128x64Actions) {
            return ((BrickletLCD128x64Actions) actions).getTouchPosition();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLCD128x64Actions class.");
        }
    }

    @RuleAction(label = "Get Touch Gesture")
    public @ActionOutput(name = "gesture", type="int")
           @ActionOutput(name = "duration", type="long")
           @ActionOutput(name = "pressureMax", type="int")
           @ActionOutput(name = "xStart", type="int")
           @ActionOutput(name = "yStart", type="int")
           @ActionOutput(name = "xEnd", type="int")
           @ActionOutput(name = "yEnd", type="int")
           @ActionOutput(name = "age", type="long")
           Map<String, Object> getTouchGesture(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletLCD128x64.TouchGesture value = ((BrickletLCD128x64)this.handler.getDevice()).getTouchGesture();
        result.put("gesture", value.gesture);
        result.put("duration", value.duration);
        result.put("pressureMax", value.pressureMax);
        result.put("xStart", value.xStart);
        result.put("yStart", value.yStart);
        result.put("xEnd", value.xEnd);
        result.put("yEnd", value.yEnd);
        result.put("age", value.age);
        return result;
    }

    public static Map<String, Object> getTouchGesture(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletLCD128x64Actions) {
            return ((BrickletLCD128x64Actions) actions).getTouchGesture();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLCD128x64Actions class.");
        }
    }

    @RuleAction(label = "Draw Line")
    public void drawLine(
            @ActionInput(name = "positionXStart") int positionXStart,
            @ActionInput(name = "positionYStart") int positionYStart,
            @ActionInput(name = "positionXEnd") int positionXEnd,
            @ActionInput(name = "positionYEnd") int positionYEnd,
            @ActionInput(name = "color") boolean color) throws TinkerforgeException {
        ((BrickletLCD128x64)this.handler.getDevice()).drawLine(positionXStart, positionYStart, positionXEnd, positionYEnd, color);
    }

    public static void drawLine(@Nullable ThingActions actions, int positionXStart, int positionYStart, int positionXEnd, int positionYEnd, boolean color) throws TinkerforgeException {
        if (actions instanceof BrickletLCD128x64Actions) {
            ((BrickletLCD128x64Actions) actions).drawLine(positionXStart, positionYStart, positionXEnd, positionYEnd, color);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLCD128x64Actions class.");
        }
    }

    @RuleAction(label = "Draw Box")
    public void drawBox(
            @ActionInput(name = "positionXStart") int positionXStart,
            @ActionInput(name = "positionYStart") int positionYStart,
            @ActionInput(name = "positionXEnd") int positionXEnd,
            @ActionInput(name = "positionYEnd") int positionYEnd,
            @ActionInput(name = "fill") boolean fill,
            @ActionInput(name = "color") boolean color) throws TinkerforgeException {
        ((BrickletLCD128x64)this.handler.getDevice()).drawBox(positionXStart, positionYStart, positionXEnd, positionYEnd, fill, color);
    }

    public static void drawBox(@Nullable ThingActions actions, int positionXStart, int positionYStart, int positionXEnd, int positionYEnd, boolean fill, boolean color) throws TinkerforgeException {
        if (actions instanceof BrickletLCD128x64Actions) {
            ((BrickletLCD128x64Actions) actions).drawBox(positionXStart, positionYStart, positionXEnd, positionYEnd, fill, color);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLCD128x64Actions class.");
        }
    }

    @RuleAction(label = "Draw Text")
    public void drawText(
            @ActionInput(name = "positionX") int positionX,
            @ActionInput(name = "positionY") int positionY,
            @ActionInput(name = "font") int font,
            @ActionInput(name = "color") boolean color,
            @ActionInput(name = "text") String text) throws TinkerforgeException {
        ((BrickletLCD128x64)this.handler.getDevice()).drawText(positionX, positionY, font, color, text);
    }

    public static void drawText(@Nullable ThingActions actions, int positionX, int positionY, int font, boolean color, String text) throws TinkerforgeException {
        if (actions instanceof BrickletLCD128x64Actions) {
            ((BrickletLCD128x64Actions) actions).drawText(positionX, positionY, font, color, text);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLCD128x64Actions class.");
        }
    }

    @RuleAction(label = "Set GUI Button")
    public void setGUIButton(
            @ActionInput(name = "index") int index,
            @ActionInput(name = "positionX") int positionX,
            @ActionInput(name = "positionY") int positionY,
            @ActionInput(name = "width") int width,
            @ActionInput(name = "height") int height,
            @ActionInput(name = "text") String text) throws TinkerforgeException {
        ((BrickletLCD128x64)this.handler.getDevice()).setGUIButton(index, positionX, positionY, width, height, text);
    }

    public static void setGUIButton(@Nullable ThingActions actions, int index, int positionX, int positionY, int width, int height, String text) throws TinkerforgeException {
        if (actions instanceof BrickletLCD128x64Actions) {
            ((BrickletLCD128x64Actions) actions).setGUIButton(index, positionX, positionY, width, height, text);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLCD128x64Actions class.");
        }
    }

    @RuleAction(label = "Get GUI Button")
    public @ActionOutput(name = "active", type="boolean")
           @ActionOutput(name = "positionX", type="int")
           @ActionOutput(name = "positionY", type="int")
           @ActionOutput(name = "width", type="int")
           @ActionOutput(name = "height", type="int")
           @ActionOutput(name = "text", type="String")
           Map<String, Object> getGUIButton(
            @ActionInput(name = "index") int index) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletLCD128x64.GUIButton value = ((BrickletLCD128x64)this.handler.getDevice()).getGUIButton(index);
        result.put("active", value.active);
        result.put("positionX", value.positionX);
        result.put("positionY", value.positionY);
        result.put("width", value.width);
        result.put("height", value.height);
        result.put("text", value.text);
        return result;
    }

    public static Map<String, Object> getGUIButton(@Nullable ThingActions actions, int index) throws TinkerforgeException {
        if (actions instanceof BrickletLCD128x64Actions) {
            return ((BrickletLCD128x64Actions) actions).getGUIButton(index);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLCD128x64Actions class.");
        }
    }

    @RuleAction(label = "Remove GUI Button")
    public void removeGUIButton(
            @ActionInput(name = "index") int index) throws TinkerforgeException {
        ((BrickletLCD128x64)this.handler.getDevice()).removeGUIButton(index);
    }

    public static void removeGUIButton(@Nullable ThingActions actions, int index) throws TinkerforgeException {
        if (actions instanceof BrickletLCD128x64Actions) {
            ((BrickletLCD128x64Actions) actions).removeGUIButton(index);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLCD128x64Actions class.");
        }
    }

    @RuleAction(label = "Get GUI Button Pressed")
    public @ActionOutput(name = "pressed", type="boolean")
           Map<String, Object> getGUIButtonPressed(
            @ActionInput(name = "index") int index) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        boolean value = ((BrickletLCD128x64)this.handler.getDevice()).getGUIButtonPressed(index);
        result.put("pressed", value);
        return result;
    }

    public static Map<String, Object> getGUIButtonPressed(@Nullable ThingActions actions, int index) throws TinkerforgeException {
        if (actions instanceof BrickletLCD128x64Actions) {
            return ((BrickletLCD128x64Actions) actions).getGUIButtonPressed(index);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLCD128x64Actions class.");
        }
    }

    @RuleAction(label = "Set GUI Slider")
    public void setGUISlider(
            @ActionInput(name = "index") int index,
            @ActionInput(name = "positionX") int positionX,
            @ActionInput(name = "positionY") int positionY,
            @ActionInput(name = "length") int length,
            @ActionInput(name = "direction") int direction,
            @ActionInput(name = "value") int value) throws TinkerforgeException {
        ((BrickletLCD128x64)this.handler.getDevice()).setGUISlider(index, positionX, positionY, length, direction, value);
    }

    public static void setGUISlider(@Nullable ThingActions actions, int index, int positionX, int positionY, int length, int direction, int value) throws TinkerforgeException {
        if (actions instanceof BrickletLCD128x64Actions) {
            ((BrickletLCD128x64Actions) actions).setGUISlider(index, positionX, positionY, length, direction, value);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLCD128x64Actions class.");
        }
    }

    @RuleAction(label = "Get GUI Slider")
    public @ActionOutput(name = "active", type="boolean")
           @ActionOutput(name = "positionX", type="int")
           @ActionOutput(name = "positionY", type="int")
           @ActionOutput(name = "length", type="int")
           @ActionOutput(name = "direction", type="int")
           @ActionOutput(name = "value", type="int")
           Map<String, Object> getGUISlider(
            @ActionInput(name = "index") int index) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletLCD128x64.GUISlider value = ((BrickletLCD128x64)this.handler.getDevice()).getGUISlider(index);
        result.put("active", value.active);
        result.put("positionX", value.positionX);
        result.put("positionY", value.positionY);
        result.put("length", value.length);
        result.put("direction", value.direction);
        result.put("value", value.value);
        return result;
    }

    public static Map<String, Object> getGUISlider(@Nullable ThingActions actions, int index) throws TinkerforgeException {
        if (actions instanceof BrickletLCD128x64Actions) {
            return ((BrickletLCD128x64Actions) actions).getGUISlider(index);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLCD128x64Actions class.");
        }
    }

    @RuleAction(label = "Remove GUI Slider")
    public void removeGUISlider(
            @ActionInput(name = "index") int index) throws TinkerforgeException {
        ((BrickletLCD128x64)this.handler.getDevice()).removeGUISlider(index);
    }

    public static void removeGUISlider(@Nullable ThingActions actions, int index) throws TinkerforgeException {
        if (actions instanceof BrickletLCD128x64Actions) {
            ((BrickletLCD128x64Actions) actions).removeGUISlider(index);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLCD128x64Actions class.");
        }
    }

    @RuleAction(label = "Get GUI Slider Value")
    public @ActionOutput(name = "value", type="int")
           Map<String, Object> getGUISliderValue(
            @ActionInput(name = "index") int index) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletLCD128x64)this.handler.getDevice()).getGUISliderValue(index);
        result.put("value", value);
        return result;
    }

    public static Map<String, Object> getGUISliderValue(@Nullable ThingActions actions, int index) throws TinkerforgeException {
        if (actions instanceof BrickletLCD128x64Actions) {
            return ((BrickletLCD128x64Actions) actions).getGUISliderValue(index);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLCD128x64Actions class.");
        }
    }

    @RuleAction(label = "Set GUI Tab Configuration")
    public void setGUITabConfiguration(
            @ActionInput(name = "changeTabConfig") int changeTabConfig,
            @ActionInput(name = "clearGUI") boolean clearGUI) throws TinkerforgeException {
        ((BrickletLCD128x64)this.handler.getDevice()).setGUITabConfiguration(changeTabConfig, clearGUI);
    }

    public static void setGUITabConfiguration(@Nullable ThingActions actions, int changeTabConfig, boolean clearGUI) throws TinkerforgeException {
        if (actions instanceof BrickletLCD128x64Actions) {
            ((BrickletLCD128x64Actions) actions).setGUITabConfiguration(changeTabConfig, clearGUI);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLCD128x64Actions class.");
        }
    }

    @RuleAction(label = "Get GUI Tab Configuration")
    public @ActionOutput(name = "changeTabConfig", type="int")
           @ActionOutput(name = "clearGUI", type="boolean")
           Map<String, Object> getGUITabConfiguration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletLCD128x64.GUITabConfiguration value = ((BrickletLCD128x64)this.handler.getDevice()).getGUITabConfiguration();
        result.put("changeTabConfig", value.changeTabConfig);
        result.put("clearGUI", value.clearGUI);
        return result;
    }

    public static Map<String, Object> getGUITabConfiguration(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletLCD128x64Actions) {
            return ((BrickletLCD128x64Actions) actions).getGUITabConfiguration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLCD128x64Actions class.");
        }
    }

    @RuleAction(label = "Set GUI Tab Text")
    public void setGUITabText(
            @ActionInput(name = "index") int index,
            @ActionInput(name = "text") String text) throws TinkerforgeException {
        ((BrickletLCD128x64)this.handler.getDevice()).setGUITabText(index, text);
    }

    public static void setGUITabText(@Nullable ThingActions actions, int index, String text) throws TinkerforgeException {
        if (actions instanceof BrickletLCD128x64Actions) {
            ((BrickletLCD128x64Actions) actions).setGUITabText(index, text);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLCD128x64Actions class.");
        }
    }

    @RuleAction(label = "Get GUI Tab Text")
    public @ActionOutput(name = "active", type="boolean")
           @ActionOutput(name = "text", type="String")
           Map<String, Object> getGUITabText(
            @ActionInput(name = "index") int index) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletLCD128x64.GUITabText value = ((BrickletLCD128x64)this.handler.getDevice()).getGUITabText(index);
        result.put("active", value.active);
        result.put("text", value.text);
        return result;
    }

    public static Map<String, Object> getGUITabText(@Nullable ThingActions actions, int index) throws TinkerforgeException {
        if (actions instanceof BrickletLCD128x64Actions) {
            return ((BrickletLCD128x64Actions) actions).getGUITabText(index);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLCD128x64Actions class.");
        }
    }

    @RuleAction(label = "Set GUI Tab Icon")
    public void setGUITabIcon(
            @ActionInput(name = "index") int index,
            @ActionInput(name = "icon") boolean[] icon) throws TinkerforgeException {
        ((BrickletLCD128x64)this.handler.getDevice()).setGUITabIcon(index, icon);
    }

    public static void setGUITabIcon(@Nullable ThingActions actions, int index, boolean[] icon) throws TinkerforgeException {
        if (actions instanceof BrickletLCD128x64Actions) {
            ((BrickletLCD128x64Actions) actions).setGUITabIcon(index, icon);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLCD128x64Actions class.");
        }
    }

    @RuleAction(label = "Get GUI Tab Icon")
    public @ActionOutput(name = "active", type="boolean")
           @ActionOutput(name = "icon", type="boolean[]")
           Map<String, Object> getGUITabIcon(
            @ActionInput(name = "index") int index) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletLCD128x64.GUITabIcon value = ((BrickletLCD128x64)this.handler.getDevice()).getGUITabIcon(index);
        result.put("active", value.active);
        result.put("icon", value.icon);
        return result;
    }

    public static Map<String, Object> getGUITabIcon(@Nullable ThingActions actions, int index) throws TinkerforgeException {
        if (actions instanceof BrickletLCD128x64Actions) {
            return ((BrickletLCD128x64Actions) actions).getGUITabIcon(index);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLCD128x64Actions class.");
        }
    }

    @RuleAction(label = "Remove GUI Tab")
    public void removeGUITab(
            @ActionInput(name = "index") int index) throws TinkerforgeException {
        ((BrickletLCD128x64)this.handler.getDevice()).removeGUITab(index);
    }

    public static void removeGUITab(@Nullable ThingActions actions, int index) throws TinkerforgeException {
        if (actions instanceof BrickletLCD128x64Actions) {
            ((BrickletLCD128x64Actions) actions).removeGUITab(index);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLCD128x64Actions class.");
        }
    }

    @RuleAction(label = "Set GUI Tab Selected")
    public void setGUITabSelected(
            @ActionInput(name = "index") int index) throws TinkerforgeException {
        ((BrickletLCD128x64)this.handler.getDevice()).setGUITabSelected(index);
    }

    public static void setGUITabSelected(@Nullable ThingActions actions, int index) throws TinkerforgeException {
        if (actions instanceof BrickletLCD128x64Actions) {
            ((BrickletLCD128x64Actions) actions).setGUITabSelected(index);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLCD128x64Actions class.");
        }
    }

    @RuleAction(label = "Get GUI Tab Selected")
    public @ActionOutput(name = "index", type="int")
           Map<String, Object> getGUITabSelected(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletLCD128x64)this.handler.getDevice()).getGUITabSelected();
        result.put("index", value);
        return result;
    }

    public static Map<String, Object> getGUITabSelected(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletLCD128x64Actions) {
            return ((BrickletLCD128x64Actions) actions).getGUITabSelected();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLCD128x64Actions class.");
        }
    }

    @RuleAction(label = "Set GUI Graph Configuration")
    public void setGUIGraphConfiguration(
            @ActionInput(name = "index") int index,
            @ActionInput(name = "graphType") int graphType,
            @ActionInput(name = "positionX") int positionX,
            @ActionInput(name = "positionY") int positionY,
            @ActionInput(name = "width") int width,
            @ActionInput(name = "height") int height,
            @ActionInput(name = "textX") String textX,
            @ActionInput(name = "textY") String textY) throws TinkerforgeException {
        ((BrickletLCD128x64)this.handler.getDevice()).setGUIGraphConfiguration(index, graphType, positionX, positionY, width, height, textX, textY);
    }

    public static void setGUIGraphConfiguration(@Nullable ThingActions actions, int index, int graphType, int positionX, int positionY, int width, int height, String textX, String textY) throws TinkerforgeException {
        if (actions instanceof BrickletLCD128x64Actions) {
            ((BrickletLCD128x64Actions) actions).setGUIGraphConfiguration(index, graphType, positionX, positionY, width, height, textX, textY);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLCD128x64Actions class.");
        }
    }

    @RuleAction(label = "Get GUI Graph Configuration")
    public @ActionOutput(name = "active", type="boolean")
           @ActionOutput(name = "graphType", type="int")
           @ActionOutput(name = "positionX", type="int")
           @ActionOutput(name = "positionY", type="int")
           @ActionOutput(name = "width", type="int")
           @ActionOutput(name = "height", type="int")
           @ActionOutput(name = "textX", type="String")
           @ActionOutput(name = "textY", type="String")
           Map<String, Object> getGUIGraphConfiguration(
            @ActionInput(name = "index") int index) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletLCD128x64.GUIGraphConfiguration value = ((BrickletLCD128x64)this.handler.getDevice()).getGUIGraphConfiguration(index);
        result.put("active", value.active);
        result.put("graphType", value.graphType);
        result.put("positionX", value.positionX);
        result.put("positionY", value.positionY);
        result.put("width", value.width);
        result.put("height", value.height);
        result.put("textX", value.textX);
        result.put("textY", value.textY);
        return result;
    }

    public static Map<String, Object> getGUIGraphConfiguration(@Nullable ThingActions actions, int index) throws TinkerforgeException {
        if (actions instanceof BrickletLCD128x64Actions) {
            return ((BrickletLCD128x64Actions) actions).getGUIGraphConfiguration(index);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLCD128x64Actions class.");
        }
    }

    @RuleAction(label = "Set GUI Graph Data")
    public void setGUIGraphData(
            @ActionInput(name = "index") int index,
            @ActionInput(name = "data") int[] data) throws TinkerforgeException {
        ((BrickletLCD128x64)this.handler.getDevice()).setGUIGraphData(index, data);
    }

    public static void setGUIGraphData(@Nullable ThingActions actions, int index, int[] data) throws TinkerforgeException {
        if (actions instanceof BrickletLCD128x64Actions) {
            ((BrickletLCD128x64Actions) actions).setGUIGraphData(index, data);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLCD128x64Actions class.");
        }
    }

    @RuleAction(label = "Get GUI Graph Data")
    public @ActionOutput(name = "data", type="int[]")
           Map<String, Object> getGUIGraphData(
            @ActionInput(name = "index") int index) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int[] value = ((BrickletLCD128x64)this.handler.getDevice()).getGUIGraphData(index);
        result.put("data", value);
        return result;
    }

    public static Map<String, Object> getGUIGraphData(@Nullable ThingActions actions, int index) throws TinkerforgeException {
        if (actions instanceof BrickletLCD128x64Actions) {
            return ((BrickletLCD128x64Actions) actions).getGUIGraphData(index);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLCD128x64Actions class.");
        }
    }

    @RuleAction(label = "Remove GUI Graph")
    public void removeGUIGraph(
            @ActionInput(name = "index") int index) throws TinkerforgeException {
        ((BrickletLCD128x64)this.handler.getDevice()).removeGUIGraph(index);
    }

    public static void removeGUIGraph(@Nullable ThingActions actions, int index) throws TinkerforgeException {
        if (actions instanceof BrickletLCD128x64Actions) {
            ((BrickletLCD128x64Actions) actions).removeGUIGraph(index);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLCD128x64Actions class.");
        }
    }

    @RuleAction(label = "Remove All GUI")
    public void removeAllGUI(
            ) throws TinkerforgeException {
        ((BrickletLCD128x64)this.handler.getDevice()).removeAllGUI();
    }

    public static void removeAllGUI(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletLCD128x64Actions) {
            ((BrickletLCD128x64Actions) actions).removeAllGUI();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLCD128x64Actions class.");
        }
    }

    @RuleAction(label = "Get Display Configuration")
    public @ActionOutput(name = "contrast", type="int")
           @ActionOutput(name = "backlight", type="int")
           @ActionOutput(name = "invert", type="boolean")
           @ActionOutput(name = "automaticDraw", type="boolean")
           Map<String, Object> getDisplayConfiguration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletLCD128x64.DisplayConfiguration value = ((BrickletLCD128x64)this.handler.getDevice()).getDisplayConfiguration();
        result.put("contrast", value.contrast);
        result.put("backlight", value.backlight);
        result.put("invert", value.invert);
        result.put("automaticDraw", value.automaticDraw);
        return result;
    }

    public static Map<String, Object> getDisplayConfiguration(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletLCD128x64Actions) {
            return ((BrickletLCD128x64Actions) actions).getDisplayConfiguration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLCD128x64Actions class.");
        }
    }

    @RuleAction(label = "Get Touch LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> getTouchLEDConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletLCD128x64)this.handler.getDevice()).getTouchLEDConfig();
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> getTouchLEDConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletLCD128x64Actions) {
            return ((BrickletLCD128x64Actions) actions).getTouchLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLCD128x64Actions class.");
        }
    }
}
