package com.tinkerforge;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletLCD20x4Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Write Line")
    public void writeLine(
            @ActionInput(name = "line") short line,
            @ActionInput(name = "position") short position,
            @ActionInput(name = "text") String text) throws TinkerforgeException {
        ((BrickletLCD20x4)this.handler.getDevice()).writeLine(line, position, text);
    }

    public static void writeLine(@Nullable ThingActions actions, short line, short position, String text) throws TinkerforgeException {
        if (actions instanceof BrickletLCD20x4Actions) {
            ((BrickletLCD20x4Actions) actions).writeLine(line, position, text);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLCD20x4Actions class.");
        }
    }

    @RuleAction(label = "Clear Display")
    public void clearDisplay(
            ) throws TinkerforgeException {
        ((BrickletLCD20x4)this.handler.getDevice()).clearDisplay();
    }

    public static void clearDisplay(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletLCD20x4Actions) {
            ((BrickletLCD20x4Actions) actions).clearDisplay();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLCD20x4Actions class.");
        }
    }

    @RuleAction(label = "Is Backlight On")
    public @ActionOutput(name = "backlight", type="boolean")
           Map<String, Object> isBacklightOn(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        boolean value = ((BrickletLCD20x4)this.handler.getDevice()).isBacklightOn();
        result.put("backlight", value);
        return result;
    }

    public static Map<String, Object> isBacklightOn(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletLCD20x4Actions) {
            return ((BrickletLCD20x4Actions) actions).isBacklightOn();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLCD20x4Actions class.");
        }
    }

    @RuleAction(label = "Get Config")
    public @ActionOutput(name = "cursor", type="boolean")
           @ActionOutput(name = "blinking", type="boolean")
           Map<String, Object> getConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletLCD20x4.Config value = ((BrickletLCD20x4)this.handler.getDevice()).getConfig();
        result.put("cursor", value.cursor);
        result.put("blinking", value.blinking);
        return result;
    }

    public static Map<String, Object> getConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletLCD20x4Actions) {
            return ((BrickletLCD20x4Actions) actions).getConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLCD20x4Actions class.");
        }
    }

    @RuleAction(label = "Is Button Pressed")
    public @ActionOutput(name = "pressed", type="boolean")
           Map<String, Object> isButtonPressed(
            @ActionInput(name = "button") short button) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        boolean value = ((BrickletLCD20x4)this.handler.getDevice()).isButtonPressed(button);
        result.put("pressed", value);
        return result;
    }

    public static Map<String, Object> isButtonPressed(@Nullable ThingActions actions, short button) throws TinkerforgeException {
        if (actions instanceof BrickletLCD20x4Actions) {
            return ((BrickletLCD20x4Actions) actions).isButtonPressed(button);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLCD20x4Actions class.");
        }
    }

    @RuleAction(label = "Set Custom Character")
    public void setCustomCharacter(
            @ActionInput(name = "index") short index,
            @ActionInput(name = "character") short[] character) throws TinkerforgeException {
        ((BrickletLCD20x4)this.handler.getDevice()).setCustomCharacter(index, character);
    }

    public static void setCustomCharacter(@Nullable ThingActions actions, short index, short[] character) throws TinkerforgeException {
        if (actions instanceof BrickletLCD20x4Actions) {
            ((BrickletLCD20x4Actions) actions).setCustomCharacter(index, character);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLCD20x4Actions class.");
        }
    }

    @RuleAction(label = "Get Custom Character")
    public @ActionOutput(name = "character", type="short[]")
           Map<String, Object> getCustomCharacter(
            @ActionInput(name = "index") short index) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        short[] value = ((BrickletLCD20x4)this.handler.getDevice()).getCustomCharacter(index);
        result.put("character", value);
        return result;
    }

    public static Map<String, Object> getCustomCharacter(@Nullable ThingActions actions, short index) throws TinkerforgeException {
        if (actions instanceof BrickletLCD20x4Actions) {
            return ((BrickletLCD20x4Actions) actions).getCustomCharacter(index);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLCD20x4Actions class.");
        }
    }

    @RuleAction(label = "Set Default Text")
    public void setDefaultText(
            @ActionInput(name = "line") short line,
            @ActionInput(name = "text") String text) throws TinkerforgeException {
        ((BrickletLCD20x4)this.handler.getDevice()).setDefaultText(line, text);
    }

    public static void setDefaultText(@Nullable ThingActions actions, short line, String text) throws TinkerforgeException {
        if (actions instanceof BrickletLCD20x4Actions) {
            ((BrickletLCD20x4Actions) actions).setDefaultText(line, text);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLCD20x4Actions class.");
        }
    }

    @RuleAction(label = "Get Default Text")
    public @ActionOutput(name = "text", type="String")
           Map<String, Object> getDefaultText(
            @ActionInput(name = "line") short line) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        String value = ((BrickletLCD20x4)this.handler.getDevice()).getDefaultText(line);
        result.put("text", value);
        return result;
    }

    public static Map<String, Object> getDefaultText(@Nullable ThingActions actions, short line) throws TinkerforgeException {
        if (actions instanceof BrickletLCD20x4Actions) {
            return ((BrickletLCD20x4Actions) actions).getDefaultText(line);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLCD20x4Actions class.");
        }
    }

    @RuleAction(label = "Set Default Text Counter")
    public void setDefaultTextCounter(
            @ActionInput(name = "counter") int counter) throws TinkerforgeException {
        ((BrickletLCD20x4)this.handler.getDevice()).setDefaultTextCounter(counter);
    }

    public static void setDefaultTextCounter(@Nullable ThingActions actions, int counter) throws TinkerforgeException {
        if (actions instanceof BrickletLCD20x4Actions) {
            ((BrickletLCD20x4Actions) actions).setDefaultTextCounter(counter);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLCD20x4Actions class.");
        }
    }

    @RuleAction(label = "Get Default Text Counter")
    public @ActionOutput(name = "counter", type="int")
           Map<String, Object> getDefaultTextCounter(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletLCD20x4)this.handler.getDevice()).getDefaultTextCounter();
        result.put("counter", value);
        return result;
    }

    public static Map<String, Object> getDefaultTextCounter(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletLCD20x4Actions) {
            return ((BrickletLCD20x4Actions) actions).getDefaultTextCounter();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLCD20x4Actions class.");
        }
    }
}
