package com.tinkerforge;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletLEDStripActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Set RGB Values")
    public void setRGBValues(
            @ActionInput(name = "index") int index,
            @ActionInput(name = "length") short length,
            @ActionInput(name = "r") short[] r,
            @ActionInput(name = "g") short[] g,
            @ActionInput(name = "b") short[] b) throws TinkerforgeException {
        ((BrickletLEDStrip)this.handler.getDevice()).setRGBValues(index, length, r, g, b);
    }

    public static void setRGBValues(@Nullable ThingActions actions, int index, short length, short[] r, short[] g, short[] b) throws TinkerforgeException {
        if (actions instanceof BrickletLEDStripActions) {
            ((BrickletLEDStripActions) actions).setRGBValues(index, length, r, g, b);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLEDStripActions class.");
        }
    }

    @RuleAction(label = "Get RGB Values")
    public @ActionOutput(name = "r", type="short[]")
           @ActionOutput(name = "g", type="short[]")
           @ActionOutput(name = "b", type="short[]")
           Map<String, Object> getRGBValues(
            @ActionInput(name = "index") int index,
            @ActionInput(name = "length") short length) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletLEDStrip.RGBValues value = ((BrickletLEDStrip)this.handler.getDevice()).getRGBValues(index, length);
        result.put("r", value.r);
        result.put("g", value.g);
        result.put("b", value.b);
        return result;
    }

    public static Map<String, Object> getRGBValues(@Nullable ThingActions actions, int index, short length) throws TinkerforgeException {
        if (actions instanceof BrickletLEDStripActions) {
            return ((BrickletLEDStripActions) actions).getRGBValues(index, length);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLEDStripActions class.");
        }
    }

    @RuleAction(label = "Get Frame Duration")
    public @ActionOutput(name = "duration", type="int")
           Map<String, Object> getFrameDuration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletLEDStrip)this.handler.getDevice()).getFrameDuration();
        result.put("duration", value);
        return result;
    }

    public static Map<String, Object> getFrameDuration(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletLEDStripActions) {
            return ((BrickletLEDStripActions) actions).getFrameDuration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLEDStripActions class.");
        }
    }

    @RuleAction(label = "Get Supply Voltage")
    public @ActionOutput(name = "voltage", type="int")
           Map<String, Object> getSupplyVoltage(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletLEDStrip)this.handler.getDevice()).getSupplyVoltage();
        result.put("voltage", value);
        return result;
    }

    public static Map<String, Object> getSupplyVoltage(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletLEDStripActions) {
            return ((BrickletLEDStripActions) actions).getSupplyVoltage();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLEDStripActions class.");
        }
    }

    @RuleAction(label = "Get Clock Frequency")
    public @ActionOutput(name = "frequency", type="long")
           Map<String, Object> getClockFrequency(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        long value = ((BrickletLEDStrip)this.handler.getDevice()).getClockFrequency();
        result.put("frequency", value);
        return result;
    }

    public static Map<String, Object> getClockFrequency(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletLEDStripActions) {
            return ((BrickletLEDStripActions) actions).getClockFrequency();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLEDStripActions class.");
        }
    }

    @RuleAction(label = "Get Chip Type")
    public @ActionOutput(name = "chip", type="int")
           Map<String, Object> getChipType(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletLEDStrip)this.handler.getDevice()).getChipType();
        result.put("chip", value);
        return result;
    }

    public static Map<String, Object> getChipType(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletLEDStripActions) {
            return ((BrickletLEDStripActions) actions).getChipType();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLEDStripActions class.");
        }
    }

    @RuleAction(label = "Set RGBW Values")
    public void setRGBWValues(
            @ActionInput(name = "index") int index,
            @ActionInput(name = "length") short length,
            @ActionInput(name = "r") short[] r,
            @ActionInput(name = "g") short[] g,
            @ActionInput(name = "b") short[] b,
            @ActionInput(name = "w") short[] w) throws TinkerforgeException {
        ((BrickletLEDStrip)this.handler.getDevice()).setRGBWValues(index, length, r, g, b, w);
    }

    public static void setRGBWValues(@Nullable ThingActions actions, int index, short length, short[] r, short[] g, short[] b, short[] w) throws TinkerforgeException {
        if (actions instanceof BrickletLEDStripActions) {
            ((BrickletLEDStripActions) actions).setRGBWValues(index, length, r, g, b, w);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLEDStripActions class.");
        }
    }

    @RuleAction(label = "Get RGBW Values")
    public @ActionOutput(name = "r", type="short[]")
           @ActionOutput(name = "g", type="short[]")
           @ActionOutput(name = "b", type="short[]")
           @ActionOutput(name = "w", type="short[]")
           Map<String, Object> getRGBWValues(
            @ActionInput(name = "index") int index,
            @ActionInput(name = "length") short length) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletLEDStrip.RGBWValues value = ((BrickletLEDStrip)this.handler.getDevice()).getRGBWValues(index, length);
        result.put("r", value.r);
        result.put("g", value.g);
        result.put("b", value.b);
        result.put("w", value.w);
        return result;
    }

    public static Map<String, Object> getRGBWValues(@Nullable ThingActions actions, int index, short length) throws TinkerforgeException {
        if (actions instanceof BrickletLEDStripActions) {
            return ((BrickletLEDStripActions) actions).getRGBWValues(index, length);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLEDStripActions class.");
        }
    }

    @RuleAction(label = "Get Channel Mapping")
    public @ActionOutput(name = "mapping", type="short")
           Map<String, Object> getChannelMapping(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        short value = ((BrickletLEDStrip)this.handler.getDevice()).getChannelMapping();
        result.put("mapping", value);
        return result;
    }

    public static Map<String, Object> getChannelMapping(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletLEDStripActions) {
            return ((BrickletLEDStripActions) actions).getChannelMapping();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLEDStripActions class.");
        }
    }
}
