package com.tinkerforge;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletLEDStripV2Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Set LED Values")
    public void setLEDValues(
            @ActionInput(name = "index") int index,
            @ActionInput(name = "value") int[] value) throws TinkerforgeException {
        ((BrickletLEDStripV2)this.handler.getDevice()).setLEDValues(index, value);
    }

    public static void setLEDValues(@Nullable ThingActions actions, int index, int[] value) throws TinkerforgeException {
        if (actions instanceof BrickletLEDStripV2Actions) {
            ((BrickletLEDStripV2Actions) actions).setLEDValues(index, value);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLEDStripV2Actions class.");
        }
    }

    @RuleAction(label = "Get LED Values")
    public @ActionOutput(name = "value", type="int[]")
           Map<String, Object> getLEDValues(
            @ActionInput(name = "index") int index,
            @ActionInput(name = "length") int length) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int[] value = ((BrickletLEDStripV2)this.handler.getDevice()).getLEDValues(index, length);
        result.put("value", value);
        return result;
    }

    public static Map<String, Object> getLEDValues(@Nullable ThingActions actions, int index, int length) throws TinkerforgeException {
        if (actions instanceof BrickletLEDStripV2Actions) {
            return ((BrickletLEDStripV2Actions) actions).getLEDValues(index, length);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLEDStripV2Actions class.");
        }
    }

    @RuleAction(label = "Get Frame Duration")
    public @ActionOutput(name = "duration", type="int")
           Map<String, Object> getFrameDuration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletLEDStripV2)this.handler.getDevice()).getFrameDuration();
        result.put("duration", value);
        return result;
    }

    public static Map<String, Object> getFrameDuration(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletLEDStripV2Actions) {
            return ((BrickletLEDStripV2Actions) actions).getFrameDuration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLEDStripV2Actions class.");
        }
    }

    @RuleAction(label = "Get Supply Voltage")
    public @ActionOutput(name = "voltage", type="int")
           Map<String, Object> getSupplyVoltage(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletLEDStripV2)this.handler.getDevice()).getSupplyVoltage();
        result.put("voltage", value);
        return result;
    }

    public static Map<String, Object> getSupplyVoltage(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletLEDStripV2Actions) {
            return ((BrickletLEDStripV2Actions) actions).getSupplyVoltage();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLEDStripV2Actions class.");
        }
    }

    @RuleAction(label = "Get Clock Frequency")
    public @ActionOutput(name = "frequency", type="long")
           Map<String, Object> getClockFrequency(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        long value = ((BrickletLEDStripV2)this.handler.getDevice()).getClockFrequency();
        result.put("frequency", value);
        return result;
    }

    public static Map<String, Object> getClockFrequency(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletLEDStripV2Actions) {
            return ((BrickletLEDStripV2Actions) actions).getClockFrequency();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLEDStripV2Actions class.");
        }
    }

    @RuleAction(label = "Get Chip Type")
    public @ActionOutput(name = "chip", type="int")
           Map<String, Object> getChipType(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletLEDStripV2)this.handler.getDevice()).getChipType();
        result.put("chip", value);
        return result;
    }

    public static Map<String, Object> getChipType(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletLEDStripV2Actions) {
            return ((BrickletLEDStripV2Actions) actions).getChipType();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLEDStripV2Actions class.");
        }
    }

    @RuleAction(label = "Get Channel Mapping")
    public @ActionOutput(name = "mapping", type="int")
           Map<String, Object> getChannelMapping(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletLEDStripV2)this.handler.getDevice()).getChannelMapping();
        result.put("mapping", value);
        return result;
    }

    public static Map<String, Object> getChannelMapping(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletLEDStripV2Actions) {
            return ((BrickletLEDStripV2Actions) actions).getChannelMapping();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletLEDStripV2Actions class.");
        }
    }
}
