package com.tinkerforge;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletMoistureActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Moisture Value")
    public @ActionOutput(name = "moisture", type="int")
           Map<String, Object> getMoistureValue(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletMoisture)this.handler.getDevice()).getMoistureValue();
        result.put("moisture", value);
        return result;
    }

    public static Map<String, Object> getMoistureValue(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletMoistureActions) {
            return ((BrickletMoistureActions) actions).getMoistureValue();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMoistureActions class.");
        }
    }

    @RuleAction(label = "Get Moving Average")
    public @ActionOutput(name = "average", type="short")
           Map<String, Object> getMovingAverage(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        short value = ((BrickletMoisture)this.handler.getDevice()).getMovingAverage();
        result.put("average", value);
        return result;
    }

    public static Map<String, Object> getMovingAverage(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletMoistureActions) {
            return ((BrickletMoistureActions) actions).getMovingAverage();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMoistureActions class.");
        }
    }
}
