package com.tinkerforge;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletMotionDetectorV2Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Motion Detected")
    public @ActionOutput(name = "motion", type="int")
           Map<String, Object> getMotionDetected(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletMotionDetectorV2)this.handler.getDevice()).getMotionDetected();
        result.put("motion", value);
        return result;
    }

    public static Map<String, Object> getMotionDetected(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletMotionDetectorV2Actions) {
            return ((BrickletMotionDetectorV2Actions) actions).getMotionDetected();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMotionDetectorV2Actions class.");
        }
    }

    @RuleAction(label = "Get Sensitivity")
    public @ActionOutput(name = "sensitivity", type="int")
           Map<String, Object> getSensitivity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletMotionDetectorV2)this.handler.getDevice()).getSensitivity();
        result.put("sensitivity", value);
        return result;
    }

    public static Map<String, Object> getSensitivity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletMotionDetectorV2Actions) {
            return ((BrickletMotionDetectorV2Actions) actions).getSensitivity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMotionDetectorV2Actions class.");
        }
    }

    @RuleAction(label = "Get Indicator")
    public @ActionOutput(name = "topLeft", type="int")
           @ActionOutput(name = "topRight", type="int")
           @ActionOutput(name = "bottom", type="int")
           Map<String, Object> getIndicator(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletMotionDetectorV2.Indicator value = ((BrickletMotionDetectorV2)this.handler.getDevice()).getIndicator();
        result.put("topLeft", value.topLeft);
        result.put("topRight", value.topRight);
        result.put("bottom", value.bottom);
        return result;
    }

    public static Map<String, Object> getIndicator(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletMotionDetectorV2Actions) {
            return ((BrickletMotionDetectorV2Actions) actions).getIndicator();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMotionDetectorV2Actions class.");
        }
    }
}
