package com.tinkerforge;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletMultiTouchV2Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Touch State")
    public @ActionOutput(name = "state", type="boolean[]")
           Map<String, Object> getTouchState(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        boolean[] value = ((BrickletMultiTouchV2)this.handler.getDevice()).getTouchState();
        result.put("state", value);
        return result;
    }

    public static Map<String, Object> getTouchState(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletMultiTouchV2Actions) {
            return ((BrickletMultiTouchV2Actions) actions).getTouchState();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMultiTouchV2Actions class.");
        }
    }

    @RuleAction(label = "Recalibrate")
    public void recalibrate(
            ) throws TinkerforgeException {
        ((BrickletMultiTouchV2)this.handler.getDevice()).recalibrate();
    }

    public static void recalibrate(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletMultiTouchV2Actions) {
            ((BrickletMultiTouchV2Actions) actions).recalibrate();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMultiTouchV2Actions class.");
        }
    }

    @RuleAction(label = "Get Electrode Config")
    public @ActionOutput(name = "enabledElectrodes", type="boolean[]")
           Map<String, Object> getElectrodeConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        boolean[] value = ((BrickletMultiTouchV2)this.handler.getDevice()).getElectrodeConfig();
        result.put("enabledElectrodes", value);
        return result;
    }

    public static Map<String, Object> getElectrodeConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletMultiTouchV2Actions) {
            return ((BrickletMultiTouchV2Actions) actions).getElectrodeConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMultiTouchV2Actions class.");
        }
    }

    @RuleAction(label = "Get Electrode Sensitivity")
    public @ActionOutput(name = "sensitivity", type="int")
           Map<String, Object> getElectrodeSensitivity(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletMultiTouchV2)this.handler.getDevice()).getElectrodeSensitivity();
        result.put("sensitivity", value);
        return result;
    }

    public static Map<String, Object> getElectrodeSensitivity(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletMultiTouchV2Actions) {
            return ((BrickletMultiTouchV2Actions) actions).getElectrodeSensitivity();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMultiTouchV2Actions class.");
        }
    }

    @RuleAction(label = "Get Touch LED Config")
    public @ActionOutput(name = "config", type="int")
           Map<String, Object> getTouchLEDConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletMultiTouchV2)this.handler.getDevice()).getTouchLEDConfig();
        result.put("config", value);
        return result;
    }

    public static Map<String, Object> getTouchLEDConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletMultiTouchV2Actions) {
            return ((BrickletMultiTouchV2Actions) actions).getTouchLEDConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletMultiTouchV2Actions class.");
        }
    }
}
