package com.tinkerforge;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletOLED64x48Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Write")
    public void write(
            @ActionInput(name = "data") short[] data) throws TinkerforgeException {
        ((BrickletOLED64x48)this.handler.getDevice()).write(data);
    }

    public static void write(@Nullable ThingActions actions, short[] data) throws TinkerforgeException {
        if (actions instanceof BrickletOLED64x48Actions) {
            ((BrickletOLED64x48Actions) actions).write(data);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletOLED64x48Actions class.");
        }
    }

    @RuleAction(label = "New Window")
    public void newWindow(
            @ActionInput(name = "columnFrom") short columnFrom,
            @ActionInput(name = "columnTo") short columnTo,
            @ActionInput(name = "rowFrom") short rowFrom,
            @ActionInput(name = "rowTo") short rowTo) throws TinkerforgeException {
        ((BrickletOLED64x48)this.handler.getDevice()).newWindow(columnFrom, columnTo, rowFrom, rowTo);
    }

    public static void newWindow(@Nullable ThingActions actions, short columnFrom, short columnTo, short rowFrom, short rowTo) throws TinkerforgeException {
        if (actions instanceof BrickletOLED64x48Actions) {
            ((BrickletOLED64x48Actions) actions).newWindow(columnFrom, columnTo, rowFrom, rowTo);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletOLED64x48Actions class.");
        }
    }

    @RuleAction(label = "Clear Display")
    public void clearDisplay(
            ) throws TinkerforgeException {
        ((BrickletOLED64x48)this.handler.getDevice()).clearDisplay();
    }

    public static void clearDisplay(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletOLED64x48Actions) {
            ((BrickletOLED64x48Actions) actions).clearDisplay();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletOLED64x48Actions class.");
        }
    }

    @RuleAction(label = "Write Line")
    public void writeLine(
            @ActionInput(name = "line") short line,
            @ActionInput(name = "position") short position,
            @ActionInput(name = "text") String text) throws TinkerforgeException {
        ((BrickletOLED64x48)this.handler.getDevice()).writeLine(line, position, text);
    }

    public static void writeLine(@Nullable ThingActions actions, short line, short position, String text) throws TinkerforgeException {
        if (actions instanceof BrickletOLED64x48Actions) {
            ((BrickletOLED64x48Actions) actions).writeLine(line, position, text);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletOLED64x48Actions class.");
        }
    }

    @RuleAction(label = "Get Display Configuration")
    public @ActionOutput(name = "contrast", type="short")
           @ActionOutput(name = "invert", type="boolean")
           Map<String, Object> getDisplayConfiguration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletOLED64x48.DisplayConfiguration value = ((BrickletOLED64x48)this.handler.getDevice()).getDisplayConfiguration();
        result.put("contrast", value.contrast);
        result.put("invert", value.invert);
        return result;
    }

    public static Map<String, Object> getDisplayConfiguration(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletOLED64x48Actions) {
            return ((BrickletOLED64x48Actions) actions).getDisplayConfiguration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletOLED64x48Actions class.");
        }
    }
}
