package com.tinkerforge;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletParticulateMatterActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get PM Concentration")
    public @ActionOutput(name = "pm10", type="int")
           @ActionOutput(name = "pm25", type="int")
           @ActionOutput(name = "pm100", type="int")
           Map<String, Object> getPMConcentration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletParticulateMatter.PMConcentration value = ((BrickletParticulateMatter)this.handler.getDevice()).getPMConcentration();
        result.put("pm10", value.pm10);
        result.put("pm25", value.pm25);
        result.put("pm100", value.pm100);
        return result;
    }

    public static Map<String, Object> getPMConcentration(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletParticulateMatterActions) {
            return ((BrickletParticulateMatterActions) actions).getPMConcentration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletParticulateMatterActions class.");
        }
    }

    @RuleAction(label = "Get PM Count")
    public @ActionOutput(name = "greater03um", type="int")
           @ActionOutput(name = "greater05um", type="int")
           @ActionOutput(name = "greater10um", type="int")
           @ActionOutput(name = "greater25um", type="int")
           @ActionOutput(name = "greater50um", type="int")
           @ActionOutput(name = "greater100um", type="int")
           Map<String, Object> getPMCount(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletParticulateMatter.PMCount value = ((BrickletParticulateMatter)this.handler.getDevice()).getPMCount();
        result.put("greater03um", value.greater03um);
        result.put("greater05um", value.greater05um);
        result.put("greater10um", value.greater10um);
        result.put("greater25um", value.greater25um);
        result.put("greater50um", value.greater50um);
        result.put("greater100um", value.greater100um);
        return result;
    }

    public static Map<String, Object> getPMCount(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletParticulateMatterActions) {
            return ((BrickletParticulateMatterActions) actions).getPMCount();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletParticulateMatterActions class.");
        }
    }

    @RuleAction(label = "Get Sensor Info")
    public @ActionOutput(name = "sensorVersion", type="int")
           @ActionOutput(name = "lastErrorCode", type="int")
           @ActionOutput(name = "framingErrorCount", type="int")
           @ActionOutput(name = "checksumErrorCount", type="int")
           Map<String, Object> getSensorInfo(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletParticulateMatter.SensorInfo value = ((BrickletParticulateMatter)this.handler.getDevice()).getSensorInfo();
        result.put("sensorVersion", value.sensorVersion);
        result.put("lastErrorCode", value.lastErrorCode);
        result.put("framingErrorCount", value.framingErrorCount);
        result.put("checksumErrorCount", value.checksumErrorCount);
        return result;
    }

    public static Map<String, Object> getSensorInfo(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletParticulateMatterActions) {
            return ((BrickletParticulateMatterActions) actions).getSensorInfo();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletParticulateMatterActions class.");
        }
    }
}
