package com.tinkerforge;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletRGBLEDButtonActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Color")
    public @ActionOutput(name = "red", type="int")
           @ActionOutput(name = "green", type="int")
           @ActionOutput(name = "blue", type="int")
           Map<String, Object> getColor(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletRGBLEDButton.Color value = ((BrickletRGBLEDButton)this.handler.getDevice()).getColor();
        result.put("red", value.red);
        result.put("green", value.green);
        result.put("blue", value.blue);
        return result;
    }

    public static Map<String, Object> getColor(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletRGBLEDButtonActions) {
            return ((BrickletRGBLEDButtonActions) actions).getColor();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRGBLEDButtonActions class.");
        }
    }

    @RuleAction(label = "Get Button State")
    public @ActionOutput(name = "state", type="int")
           Map<String, Object> getButtonState(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletRGBLEDButton)this.handler.getDevice()).getButtonState();
        result.put("state", value);
        return result;
    }

    public static Map<String, Object> getButtonState(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletRGBLEDButtonActions) {
            return ((BrickletRGBLEDButtonActions) actions).getButtonState();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRGBLEDButtonActions class.");
        }
    }

    @RuleAction(label = "Get Color Calibration")
    public @ActionOutput(name = "red", type="int")
           @ActionOutput(name = "green", type="int")
           @ActionOutput(name = "blue", type="int")
           Map<String, Object> getColorCalibration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletRGBLEDButton.ColorCalibration value = ((BrickletRGBLEDButton)this.handler.getDevice()).getColorCalibration();
        result.put("red", value.red);
        result.put("green", value.green);
        result.put("blue", value.blue);
        return result;
    }

    public static Map<String, Object> getColorCalibration(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletRGBLEDButtonActions) {
            return ((BrickletRGBLEDButtonActions) actions).getColorCalibration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRGBLEDButtonActions class.");
        }
    }
}
