package com.tinkerforge;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletRealTimeClockActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Date Time")
    public @ActionOutput(name = "year", type="int")
           @ActionOutput(name = "month", type="short")
           @ActionOutput(name = "day", type="short")
           @ActionOutput(name = "hour", type="short")
           @ActionOutput(name = "minute", type="short")
           @ActionOutput(name = "second", type="short")
           @ActionOutput(name = "centisecond", type="short")
           @ActionOutput(name = "weekday", type="short")
           Map<String, Object> getDateTime(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletRealTimeClock.DateTime value = ((BrickletRealTimeClock)this.handler.getDevice()).getDateTime();
        result.put("year", value.year);
        result.put("month", value.month);
        result.put("day", value.day);
        result.put("hour", value.hour);
        result.put("minute", value.minute);
        result.put("second", value.second);
        result.put("centisecond", value.centisecond);
        result.put("weekday", value.weekday);
        return result;
    }

    public static Map<String, Object> getDateTime(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletRealTimeClockActions) {
            return ((BrickletRealTimeClockActions) actions).getDateTime();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRealTimeClockActions class.");
        }
    }

    @RuleAction(label = "Get Timestamp")
    public @ActionOutput(name = "timestamp", type="long")
           Map<String, Object> getTimestamp(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        long value = ((BrickletRealTimeClock)this.handler.getDevice()).getTimestamp();
        result.put("timestamp", value);
        return result;
    }

    public static Map<String, Object> getTimestamp(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletRealTimeClockActions) {
            return ((BrickletRealTimeClockActions) actions).getTimestamp();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRealTimeClockActions class.");
        }
    }

    @RuleAction(label = "Get Offset")
    public @ActionOutput(name = "offset", type="byte")
           Map<String, Object> getOffset(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        byte value = ((BrickletRealTimeClock)this.handler.getDevice()).getOffset();
        result.put("offset", value);
        return result;
    }

    public static Map<String, Object> getOffset(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletRealTimeClockActions) {
            return ((BrickletRealTimeClockActions) actions).getOffset();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletRealTimeClockActions class.");
        }
    }
}
