package com.tinkerforge;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletSegmentDisplay4x7V2Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Segments")
    public @ActionOutput(name = "digit0", type="boolean[]")
           @ActionOutput(name = "digit1", type="boolean[]")
           @ActionOutput(name = "digit2", type="boolean[]")
           @ActionOutput(name = "digit3", type="boolean[]")
           @ActionOutput(name = "colon", type="boolean[]")
           @ActionOutput(name = "tick", type="boolean")
           Map<String, Object> getSegments(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletSegmentDisplay4x7V2.Segments value = ((BrickletSegmentDisplay4x7V2)this.handler.getDevice()).getSegments();
        result.put("digit0", value.digit0);
        result.put("digit1", value.digit1);
        result.put("digit2", value.digit2);
        result.put("digit3", value.digit3);
        result.put("colon", value.colon);
        result.put("tick", value.tick);
        return result;
    }

    public static Map<String, Object> getSegments(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletSegmentDisplay4x7V2Actions) {
            return ((BrickletSegmentDisplay4x7V2Actions) actions).getSegments();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletSegmentDisplay4x7V2Actions class.");
        }
    }

    @RuleAction(label = "Get Brightness")
    public @ActionOutput(name = "brightness", type="int")
           Map<String, Object> getBrightness(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletSegmentDisplay4x7V2)this.handler.getDevice()).getBrightness();
        result.put("brightness", value);
        return result;
    }

    public static Map<String, Object> getBrightness(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletSegmentDisplay4x7V2Actions) {
            return ((BrickletSegmentDisplay4x7V2Actions) actions).getBrightness();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletSegmentDisplay4x7V2Actions class.");
        }
    }

    @RuleAction(label = "Get Selected Segment")
    public @ActionOutput(name = "value", type="boolean")
           Map<String, Object> getSelectedSegment(
            @ActionInput(name = "segment") int segment) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        boolean value = ((BrickletSegmentDisplay4x7V2)this.handler.getDevice()).getSelectedSegment(segment);
        result.put("value", value);
        return result;
    }

    public static Map<String, Object> getSelectedSegment(@Nullable ThingActions actions, int segment) throws TinkerforgeException {
        if (actions instanceof BrickletSegmentDisplay4x7V2Actions) {
            return ((BrickletSegmentDisplay4x7V2Actions) actions).getSelectedSegment(segment);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletSegmentDisplay4x7V2Actions class.");
        }
    }

    @RuleAction(label = "Start Counter")
    public void startCounter(
            @ActionInput(name = "valueFrom") int valueFrom,
            @ActionInput(name = "valueTo") int valueTo,
            @ActionInput(name = "increment") int increment,
            @ActionInput(name = "length") long length) throws TinkerforgeException {
        ((BrickletSegmentDisplay4x7V2)this.handler.getDevice()).startCounter(valueFrom, valueTo, increment, length);
    }

    public static void startCounter(@Nullable ThingActions actions, int valueFrom, int valueTo, int increment, long length) throws TinkerforgeException {
        if (actions instanceof BrickletSegmentDisplay4x7V2Actions) {
            ((BrickletSegmentDisplay4x7V2Actions) actions).startCounter(valueFrom, valueTo, increment, length);
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletSegmentDisplay4x7V2Actions class.");
        }
    }

    @RuleAction(label = "Get Counter Value")
    public @ActionOutput(name = "value", type="int")
           Map<String, Object> getCounterValue(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletSegmentDisplay4x7V2)this.handler.getDevice()).getCounterValue();
        result.put("value", value);
        return result;
    }

    public static Map<String, Object> getCounterValue(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletSegmentDisplay4x7V2Actions) {
            return ((BrickletSegmentDisplay4x7V2Actions) actions).getCounterValue();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletSegmentDisplay4x7V2Actions class.");
        }
    }
}
