package com.tinkerforge;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletSolidStateRelayActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get State")
    public @ActionOutput(name = "state", type="boolean")
           Map<String, Object> getState(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        boolean value = ((BrickletSolidStateRelay)this.handler.getDevice()).getState();
        result.put("state", value);
        return result;
    }

    public static Map<String, Object> getState(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletSolidStateRelayActions) {
            return ((BrickletSolidStateRelayActions) actions).getState();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletSolidStateRelayActions class.");
        }
    }

    @RuleAction(label = "Get Monoflop")
    public @ActionOutput(name = "state", type="boolean")
           @ActionOutput(name = "time", type="long")
           @ActionOutput(name = "timeRemaining", type="long")
           Map<String, Object> getMonoflop(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletSolidStateRelay.Monoflop value = ((BrickletSolidStateRelay)this.handler.getDevice()).getMonoflop();
        result.put("state", value.state);
        result.put("time", value.time);
        result.put("timeRemaining", value.timeRemaining);
        return result;
    }

    public static Map<String, Object> getMonoflop(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletSolidStateRelayActions) {
            return ((BrickletSolidStateRelayActions) actions).getMonoflop();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletSolidStateRelayActions class.");
        }
    }
}
