package com.tinkerforge;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletTemperatureActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Temperature")
    public @ActionOutput(name = "temperature", type="short")
           Map<String, Object> getTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        short value = ((BrickletTemperature)this.handler.getDevice()).getTemperature();
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> getTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletTemperatureActions) {
            return ((BrickletTemperatureActions) actions).getTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletTemperatureActions class.");
        }
    }

    @RuleAction(label = "Get I2C Mode")
    public @ActionOutput(name = "mode", type="short")
           Map<String, Object> getI2CMode(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        short value = ((BrickletTemperature)this.handler.getDevice()).getI2CMode();
        result.put("mode", value);
        return result;
    }

    public static Map<String, Object> getI2CMode(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletTemperatureActions) {
            return ((BrickletTemperatureActions) actions).getI2CMode();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletTemperatureActions class.");
        }
    }
}
