package com.tinkerforge;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletTemperatureV2Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> getTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletTemperatureV2)this.handler.getDevice()).getTemperature();
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> getTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletTemperatureV2Actions) {
            return ((BrickletTemperatureV2Actions) actions).getTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletTemperatureV2Actions class.");
        }
    }

    @RuleAction(label = "Get Heater Configuration")
    public @ActionOutput(name = "heaterConfig", type="int")
           Map<String, Object> getHeaterConfiguration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletTemperatureV2)this.handler.getDevice()).getHeaterConfiguration();
        result.put("heaterConfig", value);
        return result;
    }

    public static Map<String, Object> getHeaterConfiguration(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletTemperatureV2Actions) {
            return ((BrickletTemperatureV2Actions) actions).getHeaterConfiguration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletTemperatureV2Actions class.");
        }
    }
}
