package com.tinkerforge;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletThermalImagingActions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get High Contrast Image")
    public @ActionOutput(name = "image", type="int[]")
           Map<String, Object> getHighContrastImage(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int[] value = ((BrickletThermalImaging)this.handler.getDevice()).getHighContrastImage();
        result.put("image", value);
        return result;
    }

    public static Map<String, Object> getHighContrastImage(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletThermalImagingActions) {
            return ((BrickletThermalImagingActions) actions).getHighContrastImage();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletThermalImagingActions class.");
        }
    }

    @RuleAction(label = "Get Temperature Image")
    public @ActionOutput(name = "image", type="int[]")
           Map<String, Object> getTemperatureImage(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int[] value = ((BrickletThermalImaging)this.handler.getDevice()).getTemperatureImage();
        result.put("image", value);
        return result;
    }

    public static Map<String, Object> getTemperatureImage(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletThermalImagingActions) {
            return ((BrickletThermalImagingActions) actions).getTemperatureImage();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletThermalImagingActions class.");
        }
    }

    @RuleAction(label = "Get Statistics")
    public @ActionOutput(name = "spotmeterStatistics", type="int[]")
           @ActionOutput(name = "temperatures", type="int[]")
           @ActionOutput(name = "resolution", type="int")
           @ActionOutput(name = "ffcStatus", type="int")
           @ActionOutput(name = "temperatureWarning", type="boolean[]")
           Map<String, Object> getStatistics(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletThermalImaging.Statistics value = ((BrickletThermalImaging)this.handler.getDevice()).getStatistics();
        result.put("spotmeterStatistics", value.spotmeterStatistics);
        result.put("temperatures", value.temperatures);
        result.put("resolution", value.resolution);
        result.put("ffcStatus", value.ffcStatus);
        result.put("temperatureWarning", value.temperatureWarning);
        return result;
    }

    public static Map<String, Object> getStatistics(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletThermalImagingActions) {
            return ((BrickletThermalImagingActions) actions).getStatistics();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletThermalImagingActions class.");
        }
    }

    @RuleAction(label = "Get Resolution")
    public @ActionOutput(name = "resolution", type="int")
           Map<String, Object> getResolution(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletThermalImaging)this.handler.getDevice()).getResolution();
        result.put("resolution", value);
        return result;
    }

    public static Map<String, Object> getResolution(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletThermalImagingActions) {
            return ((BrickletThermalImagingActions) actions).getResolution();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletThermalImagingActions class.");
        }
    }

    @RuleAction(label = "Get Spotmeter Config")
    public @ActionOutput(name = "regionOfInterest", type="int[]")
           Map<String, Object> getSpotmeterConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int[] value = ((BrickletThermalImaging)this.handler.getDevice()).getSpotmeterConfig();
        result.put("regionOfInterest", value);
        return result;
    }

    public static Map<String, Object> getSpotmeterConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletThermalImagingActions) {
            return ((BrickletThermalImagingActions) actions).getSpotmeterConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletThermalImagingActions class.");
        }
    }

    @RuleAction(label = "Get High Contrast Config")
    public @ActionOutput(name = "regionOfInterest", type="int[]")
           @ActionOutput(name = "dampeningFactor", type="int")
           @ActionOutput(name = "clipLimit", type="int[]")
           @ActionOutput(name = "emptyCounts", type="int")
           Map<String, Object> getHighContrastConfig(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletThermalImaging.HighContrastConfig value = ((BrickletThermalImaging)this.handler.getDevice()).getHighContrastConfig();
        result.put("regionOfInterest", value.regionOfInterest);
        result.put("dampeningFactor", value.dampeningFactor);
        result.put("clipLimit", value.clipLimit);
        result.put("emptyCounts", value.emptyCounts);
        return result;
    }

    public static Map<String, Object> getHighContrastConfig(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletThermalImagingActions) {
            return ((BrickletThermalImagingActions) actions).getHighContrastConfig();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletThermalImagingActions class.");
        }
    }
}
