package com.tinkerforge;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletThermocoupleV2Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Temperature")
    public @ActionOutput(name = "temperature", type="int")
           Map<String, Object> getTemperature(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletThermocoupleV2)this.handler.getDevice()).getTemperature();
        result.put("temperature", value);
        return result;
    }

    public static Map<String, Object> getTemperature(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletThermocoupleV2Actions) {
            return ((BrickletThermocoupleV2Actions) actions).getTemperature();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletThermocoupleV2Actions class.");
        }
    }

    @RuleAction(label = "Get Configuration")
    public @ActionOutput(name = "averaging", type="int")
           @ActionOutput(name = "thermocoupleType", type="int")
           @ActionOutput(name = "filter", type="int")
           Map<String, Object> getConfiguration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletThermocoupleV2.Configuration value = ((BrickletThermocoupleV2)this.handler.getDevice()).getConfiguration();
        result.put("averaging", value.averaging);
        result.put("thermocoupleType", value.thermocoupleType);
        result.put("filter", value.filter);
        return result;
    }

    public static Map<String, Object> getConfiguration(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletThermocoupleV2Actions) {
            return ((BrickletThermocoupleV2Actions) actions).getConfiguration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletThermocoupleV2Actions class.");
        }
    }

    @RuleAction(label = "Get Error State")
    public @ActionOutput(name = "overUnder", type="boolean")
           @ActionOutput(name = "openCircuit", type="boolean")
           Map<String, Object> getErrorState(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletThermocoupleV2.ErrorState value = ((BrickletThermocoupleV2)this.handler.getDevice()).getErrorState();
        result.put("overUnder", value.overUnder);
        result.put("openCircuit", value.openCircuit);
        return result;
    }

    public static Map<String, Object> getErrorState(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletThermocoupleV2Actions) {
            return ((BrickletThermocoupleV2Actions) actions).getErrorState();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletThermocoupleV2Actions class.");
        }
    }
}
