package com.tinkerforge;

import org.eclipse.jdt.annotation.NonNullByDefault;
import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.binding.tinkerforge.internal.handler.DeviceHandler;
import org.eclipse.smarthome.core.thing.binding.ThingActions;
import org.eclipse.smarthome.core.thing.binding.ThingActionsScope;
import org.eclipse.smarthome.core.thing.binding.ThingHandler;
import org.openhab.core.automation.annotation.ActionInput;
import org.openhab.core.automation.annotation.ActionOutput;
import org.openhab.core.automation.annotation.RuleAction;

import java.util.Map;
import java.util.HashMap;

@ThingActionsScope(name = "tinkerforge")
@NonNullByDefault
public class BrickletVoltageCurrentV2Actions implements ThingActions {

    private @Nullable DeviceHandler handler;

    @Override
    public void setThingHandler(@Nullable ThingHandler handler) { this.handler = (DeviceHandler) handler; }

    @Override
    public @Nullable ThingHandler getThingHandler() { return handler; }

        @RuleAction(label = "Get Voltage")
    public @ActionOutput(name = "voltage", type="int")
           Map<String, Object> getVoltage(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletVoltageCurrentV2)this.handler.getDevice()).getVoltage();
        result.put("voltage", value);
        return result;
    }

    public static Map<String, Object> getVoltage(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletVoltageCurrentV2Actions) {
            return ((BrickletVoltageCurrentV2Actions) actions).getVoltage();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletVoltageCurrentV2Actions class.");
        }
    }

    @RuleAction(label = "Get Current")
    public @ActionOutput(name = "current", type="int")
           Map<String, Object> getCurrent(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletVoltageCurrentV2)this.handler.getDevice()).getCurrent();
        result.put("current", value);
        return result;
    }

    public static Map<String, Object> getCurrent(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletVoltageCurrentV2Actions) {
            return ((BrickletVoltageCurrentV2Actions) actions).getCurrent();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletVoltageCurrentV2Actions class.");
        }
    }

    @RuleAction(label = "Get Power")
    public @ActionOutput(name = "power", type="int")
           Map<String, Object> getPower(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        int value = ((BrickletVoltageCurrentV2)this.handler.getDevice()).getPower();
        result.put("power", value);
        return result;
    }

    public static Map<String, Object> getPower(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletVoltageCurrentV2Actions) {
            return ((BrickletVoltageCurrentV2Actions) actions).getPower();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletVoltageCurrentV2Actions class.");
        }
    }

    @RuleAction(label = "Get Configuration")
    public @ActionOutput(name = "averaging", type="int")
           @ActionOutput(name = "voltageConversionTime", type="int")
           @ActionOutput(name = "currentConversionTime", type="int")
           Map<String, Object> getConfiguration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletVoltageCurrentV2.Configuration value = ((BrickletVoltageCurrentV2)this.handler.getDevice()).getConfiguration();
        result.put("averaging", value.averaging);
        result.put("voltageConversionTime", value.voltageConversionTime);
        result.put("currentConversionTime", value.currentConversionTime);
        return result;
    }

    public static Map<String, Object> getConfiguration(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletVoltageCurrentV2Actions) {
            return ((BrickletVoltageCurrentV2Actions) actions).getConfiguration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletVoltageCurrentV2Actions class.");
        }
    }

    @RuleAction(label = "Get Calibration")
    public @ActionOutput(name = "voltageMultiplier", type="int")
           @ActionOutput(name = "voltageDivisor", type="int")
           @ActionOutput(name = "currentMultiplier", type="int")
           @ActionOutput(name = "currentDivisor", type="int")
           Map<String, Object> getCalibration(
            ) throws TinkerforgeException {
        Map<String, Object> result = new HashMap<>();
        BrickletVoltageCurrentV2.Calibration value = ((BrickletVoltageCurrentV2)this.handler.getDevice()).getCalibration();
        result.put("voltageMultiplier", value.voltageMultiplier);
        result.put("voltageDivisor", value.voltageDivisor);
        result.put("currentMultiplier", value.currentMultiplier);
        result.put("currentDivisor", value.currentDivisor);
        return result;
    }

    public static Map<String, Object> getCalibration(@Nullable ThingActions actions) throws TinkerforgeException {
        if (actions instanceof BrickletVoltageCurrentV2Actions) {
            return ((BrickletVoltageCurrentV2Actions) actions).getCalibration();
        } else {
            throw new IllegalArgumentException("Instance is not an BrickletVoltageCurrentV2Actions class.");
        }
    }
}
